/*!
 * Bootstrap-select v1.13.9 (https://developer.snapappointments.com/bootstrap-select)
 *
 * Copyright 2012-2019 SnapAppointments, LLC
 * Licensed under MIT (https://github.com/snapappointments/bootstrap-select/blob/master/LICENSE)
 */

(function (root, factory) {
	if (root === undefined && window !== undefined) root = window;
	if (typeof define === 'function' && define.amd) {
		// AMD. Register as an anonymous module unless amdModuleId is set
		define(["jquery"], function (a0) {
			return (factory(a0));
		});
	} else if (typeof module === 'object' && module.exports) {
		// Node. Does not work with strict CommonJS, but
		// only CommonJS-like environments that support module.exports,
		// like Node.
		module.exports = factory(require("jquery"));
	} else {
		factory(root["jQuery"]);
	}
}(this, function (jQuery) {

(function ($) {
	'use strict';

	var DISALLOWED_ATTRIBUTES = ['sanitize', 'whiteList', 'sanitizeFn'];

	var uriAttrs = [
		'background',
		'cite',
		'href',
		'itemtype',
		'longdesc',
		'poster',
		'src',
		'xlink:href'
	];

	var ARIA_ATTRIBUTE_PATTERN = /^aria-[\w-]*$/i;

	var DefaultWhitelist = {
		// Global attributes allowed on any supplied element below.
		'*': ['class', 'dir', 'id', 'lang', 'role', 'tabindex', 'style', ARIA_ATTRIBUTE_PATTERN],
		a: ['target', 'href', 'title', 'rel'],
		area: [],
		b: [],
		br: [],
		col: [],
		code: [],
		div: [],
		em: [],
		hr: [],
		h1: [],
		h2: [],
		h3: [],
		h4: [],
		h5: [],
		h6: [],
		i: [],
		img: ['src', 'alt', 'title', 'width', 'height'],
		li: [],
		ol: [],
		p: [],
		pre: [],
		s: [],
		small: [],
		span: [],
		sub: [],
		sup: [],
		strong: [],
		u: [],
		ul: []
	}

	/**
	 * A pattern that recognizes a commonly useful subset of URLs that are safe.
	 *
	 * Shoutout to Angular 7 https://github.com/angular/angular/blob/7.2.4/packages/core/src/sanitization/url_sanitizer.ts
	 */
	var SAFE_URL_PATTERN = /^(?:(?:https?|mailto|ftp|tel|file):|[^&:/?#]*(?:[/?#]|$))/gi;

	/**
	 * A pattern that matches safe data URLs. Only matches image, video and audio types.
	 *
	 * Shoutout to Angular 7 https://github.com/angular/angular/blob/7.2.4/packages/core/src/sanitization/url_sanitizer.ts
	 */
	var DATA_URL_PATTERN = /^data:(?:image\/(?:bmp|gif|jpeg|jpg|png|tiff|webp)|video\/(?:mpeg|mp4|ogg|webm)|audio\/(?:mp3|oga|ogg|opus));base64,[a-z0-9+/]+=*$/i;

	function allowedAttribute (attr, allowedAttributeList) {
		var attrName = attr.nodeName.toLowerCase()

		if ($.inArray(attrName, allowedAttributeList) !== -1) {
			if ($.inArray(attrName, uriAttrs) !== -1) {
				return Boolean(attr.nodeValue.match(SAFE_URL_PATTERN) || attr.nodeValue.match(DATA_URL_PATTERN))
			}

			return true
		}

		var regExp = $(allowedAttributeList).filter(function (index, value) {
			return value instanceof RegExp
		})

		// Check if a regular expression validates the attribute.
		for (var i = 0, l = regExp.length; i < l; i++) {
			if (attrName.match(regExp[i])) {
				return true
			}
		}

		return false
	}

	function sanitizeHtml (unsafeElements, whiteList, sanitizeFn) {
		if (sanitizeFn && typeof sanitizeFn === 'function') {
			return sanitizeFn(unsafeElements);
		}

		var whitelistKeys = Object.keys(whiteList);

		for (var i = 0, len = unsafeElements.length; i < len; i++) {
			var elements = unsafeElements[i].querySelectorAll('*');

			for (var j = 0, len2 = elements.length; j < len2; j++) {
				var el = elements[j];
				var elName = el.nodeName.toLowerCase();

				if (whitelistKeys.indexOf(elName) === -1) {
					el.parentNode.removeChild(el);

					continue;
				}

				var attributeList = [].slice.call(el.attributes);
				var whitelistedAttributes = [].concat(whiteList['*'] || [], whiteList[elName] || []);

				for (var k = 0, len3 = attributeList.length; k < len3; k++) {
					var attr = attributeList[k];

					if (!allowedAttribute(attr, whitelistedAttributes)) {
						el.removeAttribute(attr.nodeName);
					}
				}
			}
		}
	}

	// Polyfill for browsers with no classList support
	// Remove in v2
	if (!('classList' in document.createElement('_'))) {
		(function (view) {
			if (!('Element' in view)) return;

			var classListProp = 'classList',
					protoProp = 'prototype',
					elemCtrProto = view.Element[protoProp],
					objCtr = Object,
					classListGetter = function () {
						var $elem = $(this);

						return {
							add: function (classes) {
								classes = Array.prototype.slice.call(arguments).join(' ');
								return $elem.addClass(classes);
							},
							remove: function (classes) {
								classes = Array.prototype.slice.call(arguments).join(' ');
								return $elem.removeClass(classes);
							},
							toggle: function (classes, force) {
								return $elem.toggleClass(classes, force);
							},
							contains: function (classes) {
								return $elem.hasClass(classes);
							}
						}
					};

			if (objCtr.defineProperty) {
				var classListPropDesc = {
					get: classListGetter,
					enumerable: true,
					configurable: true
				};
				try {
					objCtr.defineProperty(elemCtrProto, classListProp, classListPropDesc);
				} catch (ex) { // IE 8 doesn't support enumerable:true
					// adding undefined to fight this issue https://github.com/eligrey/classList.js/issues/36
					// modernie IE8-MSW7 machine has IE8 8.0.6001.18702 and is affected
					if (ex.number === undefined || ex.number === -0x7FF5EC54) {
						classListPropDesc.enumerable = false;
						objCtr.defineProperty(elemCtrProto, classListProp, classListPropDesc);
					}
				}
			} else if (objCtr[protoProp].__defineGetter__) {
				elemCtrProto.__defineGetter__(classListProp, classListGetter);
			}
		}(window));
	}

	var testElement = document.createElement('_');

	testElement.classList.add('c1', 'c2');

	if (!testElement.classList.contains('c2')) {
		var _add = DOMTokenList.prototype.add,
				_remove = DOMTokenList.prototype.remove;

		DOMTokenList.prototype.add = function () {
			Array.prototype.forEach.call(arguments, _add.bind(this));
		}

		DOMTokenList.prototype.remove = function () {
			Array.prototype.forEach.call(arguments, _remove.bind(this));
		}
	}

	testElement.classList.toggle('c3', false);

	// Polyfill for IE 10 and Firefox <24, where classList.toggle does not
	// support the second argument.
	if (testElement.classList.contains('c3')) {
		var _toggle = DOMTokenList.prototype.toggle;

		DOMTokenList.prototype.toggle = function (token, force) {
			if (1 in arguments && !this.contains(token) === !force) {
				return force;
			} else {
				return _toggle.call(this, token);
			}
		};
	}

	testElement = null;

	// shallow array comparison
	function isEqual (array1, array2) {
		return array1.length === array2.length && array1.every(function (element, index) {
			return element === array2[index];
		});
	};

	// <editor-fold desc="Shims">
	if (!String.prototype.startsWith) {
		(function () {
			'use strict';
			var defineProperty = (function () {
				try {
					var object = {};
					var $defineProperty = Object.defineProperty;
					var result = $defineProperty(object, object, object) && $defineProperty;
				} catch (error) {
				}
				return result;
			}());
			var toString = {}.toString;
			var startsWith = function (search) {
				if (this == null) {
					throw new TypeError();
				}
				var string = String(this);
				if (search && toString.call(search) == '[object RegExp]') {
					throw new TypeError();
				}
				var stringLength = string.length;
				var searchString = String(search);
				var searchLength = searchString.length;
				var position = arguments.length > 1 ? arguments[1] : undefined;
				var pos = position ? Number(position) : 0;
				if (pos != pos) {
					pos = 0;
				}
				var start = Math.min(Math.max(pos, 0), stringLength);
				if (searchLength + start > stringLength) {
					return false;
				}
				var index = -1;
				while (++index < searchLength) {
					if (string.charCodeAt(start + index) != searchString.charCodeAt(index)) {
						return false;
					}
				}
				return true;
			};
			if (defineProperty) {
				defineProperty(String.prototype, 'startsWith', {
					'value': startsWith,
					'configurable': true,
					'writable': true
				});
			} else {
				String.prototype.startsWith = startsWith;
			}
		}());
	}

	if (!Object.keys) {
		Object.keys = function (
			o, // object
			k, // key
			r  // result array
		) {
			// initialize object and result
			r = [];
			// iterate over object keys
			for (k in o) {
				// fill result array with non-prototypical keys
				r.hasOwnProperty.call(o, k) && r.push(k);
			}
			// return result
			return r;
		};
	}

	if (HTMLSelectElement && !HTMLSelectElement.prototype.hasOwnProperty('selectedOptions')) {
		Object.defineProperty(HTMLSelectElement.prototype, 'selectedOptions', {
			get: function () {
				return this.querySelectorAll(':checked');
			}
		});
	}

	// much faster than $.val()
	function getSelectValues (select) {
		var result = [];
		var options = select.selectedOptions;
		var opt;

		if (select.multiple) {
			for (var i = 0, len = options.length; i < len; i++) {
				opt = options[i];

				result.push(opt.value || opt.text);
			}
		} else {
			result = select.value;
		}

		return result;
	}

	// set data-selected on select element if the value has been programmatically selected
	// prior to initialization of bootstrap-select
	// * consider removing or replacing an alternative method *
	var valHooks = {
		useDefault: false,
		_set: $.valHooks.select.set
	};

	$.valHooks.select.set = function (elem, value) {
		if (value && !valHooks.useDefault) $(elem).data('selected', true);

		return valHooks._set.apply(this, arguments);
	};

	var changedArguments = null;

	var EventIsSupported = (function () {
		try {
			new Event('change');
			return true;
		} catch (e) {
			return false;
		}
	})();

	$.fn.triggerNative = function (eventName) {
		var el = this[0],
				event;

		if (el.dispatchEvent) { // for modern browsers & IE9+
			if (EventIsSupported) {
				// For modern browsers
				event = new Event(eventName, {
					bubbles: true
				});
			} else {
				// For IE since it doesn't support Event constructor
				event = document.createEvent('Event');
				event.initEvent(eventName, true, false);
			}

			el.dispatchEvent(event);
		} else if (el.fireEvent) { // for IE8
			event = document.createEventObject();
			event.eventType = eventName;
			el.fireEvent('on' + eventName, event);
		} else {
			// fall back to jQuery.trigger
			this.trigger(eventName);
		}
	};
	// </editor-fold>

	function stringSearch (li, searchString, method, normalize) {
		var stringTypes = [
					'display',
					'subtext',
					'tokens'
				],
				searchSuccess = false;

		for (var i = 0; i < stringTypes.length; i++) {
			var stringType = stringTypes[i],
					string = li[stringType];

			if (string) {
				string = string.toString();

				// Strip HTML tags. This isn't perfect, but it's much faster than any other method
				if (stringType === 'display') {
					string = string.replace(/<[^>]+>/g, '');
				}

				if (normalize) string = normalizeToBase(string);
				string = string.toUpperCase();

				if (method === 'contains') {
					searchSuccess = string.indexOf(searchString) >= 0;
				} else {
					searchSuccess = string.startsWith(searchString);
				}

				if (searchSuccess) break;
			}
		}

		return searchSuccess;
	}

	function toInteger (value) {
		return parseInt(value, 10) || 0;
	}

	// Borrowed from Lodash (_.deburr)
	/** Used to map Latin Unicode letters to basic Latin letters. */
	var deburredLetters = {
		// Latin-1 Supplement block.
		'\xc0': 'A',  '\xc1': 'A', '\xc2': 'A', '\xc3': 'A', '\xc4': 'A', '\xc5': 'A',
		'\xe0': 'a',  '\xe1': 'a', '\xe2': 'a', '\xe3': 'a', '\xe4': 'a', '\xe5': 'a',
		'\xc7': 'C',  '\xe7': 'c',
		'\xd0': 'D',  '\xf0': 'd',
		'\xc8': 'E',  '\xc9': 'E', '\xca': 'E', '\xcb': 'E',
		'\xe8': 'e',  '\xe9': 'e', '\xea': 'e', '\xeb': 'e',
		'\xcc': 'I',  '\xcd': 'I', '\xce': 'I', '\xcf': 'I',
		'\xec': 'i',  '\xed': 'i', '\xee': 'i', '\xef': 'i',
		'\xd1': 'N',  '\xf1': 'n',
		'\xd2': 'O',  '\xd3': 'O', '\xd4': 'O', '\xd5': 'O', '\xd6': 'O', '\xd8': 'O',
		'\xf2': 'o',  '\xf3': 'o', '\xf4': 'o', '\xf5': 'o', '\xf6': 'o', '\xf8': 'o',
		'\xd9': 'U',  '\xda': 'U', '\xdb': 'U', '\xdc': 'U',
		'\xf9': 'u',  '\xfa': 'u', '\xfb': 'u', '\xfc': 'u',
		'\xdd': 'Y',  '\xfd': 'y', '\xff': 'y',
		'\xc6': 'Ae', '\xe6': 'ae',
		'\xde': 'Th', '\xfe': 'th',
		'\xdf': 'ss',
		// Latin Extended-A block.
		'\u0100': 'A',  '\u0102': 'A', '\u0104': 'A',
		'\u0101': 'a',  '\u0103': 'a', '\u0105': 'a',
		'\u0106': 'C',  '\u0108': 'C', '\u010a': 'C', '\u010c': 'C',
		'\u0107': 'c',  '\u0109': 'c', '\u010b': 'c', '\u010d': 'c',
		'\u010e': 'D',  '\u0110': 'D', '\u010f': 'd', '\u0111': 'd',
		'\u0112': 'E',  '\u0114': 'E', '\u0116': 'E', '\u0118': 'E', '\u011a': 'E',
		'\u0113': 'e',  '\u0115': 'e', '\u0117': 'e', '\u0119': 'e', '\u011b': 'e',
		'\u011c': 'G',  '\u011e': 'G', '\u0120': 'G', '\u0122': 'G',
		'\u011d': 'g',  '\u011f': 'g', '\u0121': 'g', '\u0123': 'g',
		'\u0124': 'H',  '\u0126': 'H', '\u0125': 'h', '\u0127': 'h',
		'\u0128': 'I',  '\u012a': 'I', '\u012c': 'I', '\u012e': 'I', '\u0130': 'I',
		'\u0129': 'i',  '\u012b': 'i', '\u012d': 'i', '\u012f': 'i', '\u0131': 'i',
		'\u0134': 'J',  '\u0135': 'j',
		'\u0136': 'K',  '\u0137': 'k', '\u0138': 'k',
		'\u0139': 'L',  '\u013b': 'L', '\u013d': 'L', '\u013f': 'L', '\u0141': 'L',
		'\u013a': 'l',  '\u013c': 'l', '\u013e': 'l', '\u0140': 'l', '\u0142': 'l',
		'\u0143': 'N',  '\u0145': 'N', '\u0147': 'N', '\u014a': 'N',
		'\u0144': 'n',  '\u0146': 'n', '\u0148': 'n', '\u014b': 'n',
		'\u014c': 'O',  '\u014e': 'O', '\u0150': 'O',
		'\u014d': 'o',  '\u014f': 'o', '\u0151': 'o',
		'\u0154': 'R',  '\u0156': 'R', '\u0158': 'R',
		'\u0155': 'r',  '\u0157': 'r', '\u0159': 'r',
		'\u015a': 'S',  '\u015c': 'S', '\u015e': 'S', '\u0160': 'S',
		'\u015b': 's',  '\u015d': 's', '\u015f': 's', '\u0161': 's',
		'\u0162': 'T',  '\u0164': 'T', '\u0166': 'T',
		'\u0163': 't',  '\u0165': 't', '\u0167': 't',
		'\u0168': 'U',  '\u016a': 'U', '\u016c': 'U', '\u016e': 'U', '\u0170': 'U', '\u0172': 'U',
		'\u0169': 'u',  '\u016b': 'u', '\u016d': 'u', '\u016f': 'u', '\u0171': 'u', '\u0173': 'u',
		'\u0174': 'W',  '\u0175': 'w',
		'\u0176': 'Y',  '\u0177': 'y', '\u0178': 'Y',
		'\u0179': 'Z',  '\u017b': 'Z', '\u017d': 'Z',
		'\u017a': 'z',  '\u017c': 'z', '\u017e': 'z',
		'\u0132': 'IJ', '\u0133': 'ij',
		'\u0152': 'Oe', '\u0153': 'oe',
		'\u0149': "'n", '\u017f': 's'
	};

	/** Used to match Latin Unicode letters (excluding mathematical operators). */
	var reLatin = /[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g;

	/** Used to compose unicode character classes. */
	var rsComboMarksRange = '\\u0300-\\u036f',
			reComboHalfMarksRange = '\\ufe20-\\ufe2f',
			rsComboSymbolsRange = '\\u20d0-\\u20ff',
			rsComboMarksExtendedRange = '\\u1ab0-\\u1aff',
			rsComboMarksSupplementRange = '\\u1dc0-\\u1dff',
			rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange + rsComboMarksExtendedRange + rsComboMarksSupplementRange;

	/** Used to compose unicode capture groups. */
	var rsCombo = '[' + rsComboRange + ']';

	/**
	 * Used to match [combining diacritical marks](https://en.wikipedia.org/wiki/Combining_Diacritical_Marks) and
	 * [combining diacritical marks for symbols](https://en.wikipedia.org/wiki/Combining_Diacritical_Marks_for_Symbols).
	 */
	var reComboMark = RegExp(rsCombo, 'g');

	function deburrLetter (key) {
		return deburredLetters[key];
	};

	function normalizeToBase (string) {
		string = string.toString();
		return string && string.replace(reLatin, deburrLetter).replace(reComboMark, '');
	}

	// List of HTML entities for escaping.
	var escapeMap = {
		'&': '&amp;',
		'<': '&lt;',
		'>': '&gt;',
		'"': '&quot;',
		"'": '&#x27;',
	};
	escapeMap[String.fromCharCode(96)] = '&#x60;';

	// Functions for escaping and unescaping strings to/from HTML interpolation.
	var createEscaper = function (map) {
		var escaper = function (match) {
			return map[match];
		};
		// Regexes for identifying a key that needs to be escaped.
		var source = '(?:' + Object.keys(map).join('|') + ')';
		var testRegexp = RegExp(source);
		var replaceRegexp = RegExp(source, 'g');
		return function (string) {
			string = string == null ? '' : '' + string;
			return testRegexp.test(string) ? string.replace(replaceRegexp, escaper) : string;
		};
	};

	var htmlEscape = createEscaper(escapeMap);

	/**
	 * ------------------------------------------------------------------------
	 * Constants
	 * ------------------------------------------------------------------------
	 */

	var keyCodeMap = {
		32: ' ',
		48: '0',
		49: '1',
		50: '2',
		51: '3',
		52: '4',
		53: '5',
		54: '6',
		55: '7',
		56: '8',
		57: '9',
		59: ';',
		65: 'A',
		66: 'B',
		67: 'C',
		68: 'D',
		69: 'E',
		70: 'F',
		71: 'G',
		72: 'H',
		73: 'I',
		74: 'J',
		75: 'K',
		76: 'L',
		77: 'M',
		78: 'N',
		79: 'O',
		80: 'P',
		81: 'Q',
		82: 'R',
		83: 'S',
		84: 'T',
		85: 'U',
		86: 'V',
		87: 'W',
		88: 'X',
		89: 'Y',
		90: 'Z',
		96: '0',
		97: '1',
		98: '2',
		99: '3',
		100: '4',
		101: '5',
		102: '6',
		103: '7',
		104: '8',
		105: '9'
	};

	var keyCodes = {
		ESCAPE: 27, // KeyboardEvent.which value for Escape (Esc) key
		ENTER: 13, // KeyboardEvent.which value for Enter key
		SPACE: 32, // KeyboardEvent.which value for space key
		TAB: 9, // KeyboardEvent.which value for tab key
		ARROW_UP: 38, // KeyboardEvent.which value for up arrow key
		ARROW_DOWN: 40 // KeyboardEvent.which value for down arrow key
	}

	var version = {
		success: false,
		major: '3'
	};

	try {
		version.full = ($.fn.dropdown.Constructor.VERSION || '').split(' ')[0].split('.');
		version.major = version.full[0];
		version.success = true;
	} catch (err) {
		// do nothing
	}

	var selectId = 0;

	var EVENT_KEY = '.bs.select';

	var classNames = {
		DISABLED: 'disabled',
		DIVIDER: 'divider',
		SHOW: 'open',
		DROPUP: 'dropup',
		MENU: 'dropdown-menu',
		MENURIGHT: 'dropdown-menu-right',
		MENULEFT: 'dropdown-menu-left',
		// to-do: replace with more advanced template/customization options
		BUTTONCLASS: 'btn-default',
		POPOVERHEADER: 'popover-title',
		ICONBASE: 'glyphicon',
		TICKICON: 'glyphicon-ok'
	}

	var Selector = {
		MENU: '.' + classNames.MENU
	}

	var elementTemplates = {
		span: document.createElement('span'),
		i: document.createElement('i'),
		subtext: document.createElement('small'),
		a: document.createElement('a'),
		li: document.createElement('li'),
		whitespace: document.createTextNode('\u00A0'),
		fragment: document.createDocumentFragment()
	}

	elementTemplates.a.setAttribute('role', 'option');
	elementTemplates.subtext.className = 'text-muted';

	elementTemplates.text = elementTemplates.span.cloneNode(false);
	elementTemplates.text.className = 'text';

	elementTemplates.checkMark = elementTemplates.span.cloneNode(false);

	var REGEXP_ARROW = new RegExp(keyCodes.ARROW_UP + '|' + keyCodes.ARROW_DOWN);
	var REGEXP_TAB_OR_ESCAPE = new RegExp('^' + keyCodes.TAB + '$|' + keyCodes.ESCAPE);

	var generateOption = {
		li: function (content, classes, optgroup) {
			var li = elementTemplates.li.cloneNode(false);

			if (content) {
				if (content.nodeType === 1 || content.nodeType === 11) {
					li.appendChild(content);
				} else {
					li.innerHTML = content;
				}
			}

			if (typeof classes !== 'undefined' && classes !== '') li.className = classes;
			if (typeof optgroup !== 'undefined' && optgroup !== null) li.classList.add('optgroup-' + optgroup);

			return li;
		},

		a: function (text, classes, inline) {
			var a = elementTemplates.a.cloneNode(true);

			if (text) {
				if (text.nodeType === 11) {
					a.appendChild(text);
				} else {
					a.insertAdjacentHTML('beforeend', text);
				}
			}

			if (typeof classes !== 'undefined' && classes !== '') a.className = classes;
			if (version.major === '4') a.classList.add('dropdown-item');
			if (inline) a.setAttribute('style', inline);

			return a;
		},

		text: function (options, useFragment) {
			var textElement = elementTemplates.text.cloneNode(false),
					subtextElement,
					iconElement;

			if (options.content) {
				textElement.innerHTML = options.content;
			} else {
				textElement.textContent = options.text;

				if (options.icon) {
					var whitespace = elementTemplates.whitespace.cloneNode(false);

					// need to use <i> for icons in the button to prevent a breaking change
					// note: switch to span in next major release
					iconElement = (useFragment === true ? elementTemplates.i : elementTemplates.span).cloneNode(false);
					iconElement.className = options.iconBase + ' ' + options.icon;

					elementTemplates.fragment.appendChild(iconElement);
					elementTemplates.fragment.appendChild(whitespace);
				}

				if (options.subtext) {
					subtextElement = elementTemplates.subtext.cloneNode(false);
					subtextElement.textContent = options.subtext;
					textElement.appendChild(subtextElement);
				}
			}

			if (useFragment === true) {
				while (textElement.childNodes.length > 0) {
					elementTemplates.fragment.appendChild(textElement.childNodes[0]);
				}
			} else {
				elementTemplates.fragment.appendChild(textElement);
			}

			return elementTemplates.fragment;
		},

		label: function (options) {
			var textElement = elementTemplates.text.cloneNode(false),
					subtextElement,
					iconElement;

			textElement.innerHTML = options.label;

			if (options.icon) {
				var whitespace = elementTemplates.whitespace.cloneNode(false);

				iconElement = elementTemplates.span.cloneNode(false);
				iconElement.className = options.iconBase + ' ' + options.icon;

				elementTemplates.fragment.appendChild(iconElement);
				elementTemplates.fragment.appendChild(whitespace);
			}

			if (options.subtext) {
				subtextElement = elementTemplates.subtext.cloneNode(false);
				subtextElement.textContent = options.subtext;
				textElement.appendChild(subtextElement);
			}

			elementTemplates.fragment.appendChild(textElement);

			return elementTemplates.fragment;
		}
	}

	var Selectpicker = function (element, options) {
		var that = this;

		// bootstrap-select has been initialized - revert valHooks.select.set back to its original function
		if (!valHooks.useDefault) {
			$.valHooks.select.set = valHooks._set;
			valHooks.useDefault = true;
		}

		this.$element = $(element);
		this.$newElement = null;
		this.$button = null;
		this.$menu = null;
		this.options = options;
		this.selectpicker = {
			main: {},
			current: {}, // current changes if a search is in progress
			search: {},
			view: {},
			keydown: {
				keyHistory: '',
				resetKeyHistory: {
					start: function () {
						return setTimeout(function () {
							that.selectpicker.keydown.keyHistory = '';
						}, 800);
					}
				}
			}
		};
		// If we have no title yet, try to pull it from the html title attribute (jQuery doesnt' pick it up as it's not a
		// data-attribute)
		if (this.options.title === null) {
			this.options.title = this.$element.attr('title');
		}

		// Format window padding
		var winPad = this.options.windowPadding;
		if (typeof winPad === 'number') {
			this.options.windowPadding = [winPad, winPad, winPad, winPad];
		}

		// Expose public methods
		this.val = Selectpicker.prototype.val;
		this.render = Selectpicker.prototype.render;
		this.refresh = Selectpicker.prototype.refresh;
		this.setStyle = Selectpicker.prototype.setStyle;
		this.selectAll = Selectpicker.prototype.selectAll;
		this.deselectAll = Selectpicker.prototype.deselectAll;
		this.destroy = Selectpicker.prototype.destroy;
		this.remove = Selectpicker.prototype.remove;
		this.show = Selectpicker.prototype.show;
		this.hide = Selectpicker.prototype.hide;

		this.init();
	};

	Selectpicker.VERSION = '1.13.9';

	// part of this is duplicated in i18n/defaults-en_US.js. Make sure to update both.
	Selectpicker.DEFAULTS = {
		noneSelectedText: 'Nothing selected',
		noneResultsText: 'No results matched {0}',
		countSelectedText: function (numSelected, numTotal) {
			return (numSelected == 1) ? '{0} item selected' : '{0} items selected';
		},
		maxOptionsText: function (numAll, numGroup) {
			return [
				(numAll == 1) ? 'Limit reached ({n} item max)' : 'Limit reached ({n} items max)',
				(numGroup == 1) ? 'Group limit reached ({n} item max)' : 'Group limit reached ({n} items max)'
			];
		},
		selectAllText: 'Select All',
		deselectAllText: 'Deselect All',
		doneButton: false,
		doneButtonText: 'Close',
		multipleSeparator: ', ',
		styleBase: 'btn',
		style: classNames.BUTTONCLASS,
		size: 'auto',
		title: null,
		selectedTextFormat: 'values',
		width: false,
		container: false,
		hideDisabled: false,
		showSubtext: false,
		showIcon: true,
		showContent: true,
		dropupAuto: true,
		header: false,
		liveSearch: false,
		liveSearchPlaceholder: null,
		liveSearchNormalize: false,
		liveSearchStyle: 'contains',
		actionsBox: false,
		iconBase: classNames.ICONBASE,
		tickIcon: classNames.TICKICON,
		showTick: false,
		template: {
			caret: '<span class="caret"></span>'
		},
		maxOptions: false,
		mobile: false,
		selectOnTab: false,
		dropdownAlignRight: false,
		windowPadding: 0,
		virtualScroll: 600,
		display: false,
		sanitize: true,
		sanitizeFn: null,
		whiteList: DefaultWhitelist
	};

	Selectpicker.prototype = {

		constructor: Selectpicker,

		init: function () {
			var that = this,
					id = this.$element.attr('id');

			this.selectId = selectId++;

			this.$element[0].classList.add('bs-select-hidden');

			this.multiple = this.$element.prop('multiple');
			this.autofocus = this.$element.prop('autofocus');
			this.options.showTick = this.$element[0].classList.contains('show-tick');

			this.$newElement = this.createDropdown();
			this.$element
				.after(this.$newElement)
				.prependTo(this.$newElement);

			this.$button = this.$newElement.children('button');
			this.$menu = this.$newElement.children(Selector.MENU);
			this.$menuInner = this.$menu.children('.inner');
			this.$searchbox = this.$menu.find('input');

			this.$element[0].classList.remove('bs-select-hidden');

			if (this.options.dropdownAlignRight === true) this.$menu[0].classList.add(classNames.MENURIGHT);

			if (typeof id !== 'undefined') {
				this.$button.attr('data-id', id);
			}

			this.checkDisabled();
			this.clickListener();
			if (this.options.liveSearch) this.liveSearchListener();
			this.setStyle();
			this.render();
			this.setWidth();
			if (this.options.container) {
				this.selectPosition();
			} else {
				this.$element.on('hide' + EVENT_KEY, function () {
					if (that.isVirtual()) {
						// empty menu on close
						var menuInner = that.$menuInner[0],
								emptyMenu = menuInner.firstChild.cloneNode(false);

						// replace the existing UL with an empty one - this is faster than $.empty() or innerHTML = ''
						menuInner.replaceChild(emptyMenu, menuInner.firstChild);
						menuInner.scrollTop = 0;
					}
				});
			}
			this.$menu.data('this', this);
			this.$newElement.data('this', this);
			if (this.options.mobile) this.mobile();

			this.$newElement.on({
				'hide.bs.dropdown': function (e) {
					that.$menuInner.attr('aria-expanded', false);
					that.$element.trigger('hide' + EVENT_KEY, e);
				},
				'hidden.bs.dropdown': function (e) {
					that.$element.trigger('hidden' + EVENT_KEY, e);
				},
				'show.bs.dropdown': function (e) {
					that.$menuInner.attr('aria-expanded', true);
					that.$element.trigger('show' + EVENT_KEY, e);
				},
				'shown.bs.dropdown': function (e) {
					that.$element.trigger('shown' + EVENT_KEY, e);
				}
			});

			if (that.$element[0].hasAttribute('required')) {
				this.$element.on('invalid' + EVENT_KEY, function () {
					that.$button[0].classList.add('bs-invalid');

					that.$element
						.on('shown' + EVENT_KEY + '.invalid', function () {
							that.$element
								.val(that.$element.val()) // set the value to hide the validation message in Chrome when menu is opened
								.off('shown' + EVENT_KEY + '.invalid');
						})
						.on('rendered' + EVENT_KEY, function () {
							// if select is no longer invalid, remove the bs-invalid class
							if (this.validity.valid) that.$button[0].classList.remove('bs-invalid');
							that.$element.off('rendered' + EVENT_KEY);
						});

					that.$button.on('blur' + EVENT_KEY, function () {
						that.$element.trigger('focus').trigger('blur');
						that.$button.off('blur' + EVENT_KEY);
					});
				});
			}

			setTimeout(function () {
				that.createLi();
				that.$element.trigger('loaded' + EVENT_KEY);
			});
		},

		createDropdown: function () {
			// Options
			// If we are multiple or showTick option is set, then add the show-tick class
			var showTick = (this.multiple || this.options.showTick) ? ' show-tick' : '',
					inputGroup = '',
					autofocus = this.autofocus ? ' autofocus' : '';

			if (version.major < 4 && this.$element.parent().hasClass('input-group')) {
				inputGroup = ' input-group-btn';
			}

			// Elements
			var drop,
					header = '',
					searchbox = '',
					actionsbox = '',
					donebutton = '';

			if (this.options.header) {
				header =
					'<div class="' + classNames.POPOVERHEADER + '">' +
						'<button type="button" class="close" aria-hidden="true">&times;</button>' +
							this.options.header +
					'</div>';
			}

			if (this.options.liveSearch) {
				searchbox =
					'<div class="bs-searchbox">' +
						'<input type="text" class="form-control" autocomplete="off"' +
							(
								this.options.liveSearchPlaceholder === null ? ''
								:
								' placeholder="' + htmlEscape(this.options.liveSearchPlaceholder) + '"'
							) +
							' role="textbox" aria-label="Search">' +
					'</div>';
			}

			if (this.multiple && this.options.actionsBox) {
				actionsbox =
					'<div class="bs-actionsbox">' +
						'<div class="btn-group btn-group-sm btn-block">' +
							'<button type="button" class="actions-btn bs-select-all btn ' + classNames.BUTTONCLASS + '">' +
								this.options.selectAllText +
							'</button>' +
							'<button type="button" class="actions-btn bs-deselect-all btn ' + classNames.BUTTONCLASS + '">' +
								this.options.deselectAllText +
							'</button>' +
						'</div>' +
					'</div>';
			}

			if (this.multiple && this.options.doneButton) {
				donebutton =
					'<div class="bs-donebutton">' +
						'<div class="btn-group btn-block">' +
							'<button type="button" class="btn btn-sm ' + classNames.BUTTONCLASS + '">' +
								this.options.doneButtonText +
							'</button>' +
						'</div>' +
					'</div>';
			}

			drop =
				'<div class="dropdown bootstrap-select' + showTick + inputGroup + '">' +
					'<button type="button" class="' + this.options.styleBase + ' dropdown-toggle" ' + (this.options.display === 'static' ? 'data-display="static"' : '') + 'data-toggle="dropdown"' + autofocus + ' role="button">' +
						'<div class="filter-option">' +
							'<div class="filter-option-inner">' +
								'<div class="filter-option-inner-inner"></div>' +
							'</div> ' +
						'</div>' +
						(
							version.major === '4' ? ''
							:
							'<span class="bs-caret">' +
								this.options.template.caret +
							'</span>'
						) +
					'</button>' +
					'<div class="' + classNames.MENU + ' ' + (version.major === '4' ? '' : classNames.SHOW) + '" role="combobox">' +
						header +
						searchbox +
						actionsbox +
						'<div class="inner ' + classNames.SHOW + '" role="listbox" aria-expanded="false" tabindex="-1">' +
								'<ul class="' + classNames.MENU + ' inner ' + (version.major === '4' ? classNames.SHOW : '') + '">' +
								'</ul>' +
						'</div>' +
						donebutton +
					'</div>' +
				'</div>';

			return $(drop);
		},

		setPositionData: function () {
			this.selectpicker.view.canHighlight = [];

			for (var i = 0; i < this.selectpicker.current.data.length; i++) {
				var li = this.selectpicker.current.data[i],
						canHighlight = true;

				if (li.type === 'divider') {
					canHighlight = false;
					li.height = this.sizeInfo.dividerHeight;
				} else if (li.type === 'optgroup-label') {
					canHighlight = false;
					li.height = this.sizeInfo.dropdownHeaderHeight;
				} else {
					li.height = this.sizeInfo.liHeight;
				}

				if (li.disabled) canHighlight = false;

				this.selectpicker.view.canHighlight.push(canHighlight);

				li.position = (i === 0 ? 0 : this.selectpicker.current.data[i - 1].position) + li.height;
			}
		},

		isVirtual: function () {
			return (this.options.virtualScroll !== false) && (this.selectpicker.main.elements.length >= this.options.virtualScroll) || this.options.virtualScroll === true;
		},

		createView: function (isSearching, scrollTop) {
			scrollTop = scrollTop || 0;

			var that = this;

			this.selectpicker.current = isSearching ? this.selectpicker.search : this.selectpicker.main;

			var active = [];
			var selected;
			var prevActive;

			this.setPositionData();

			scroll(scrollTop, true);

			this.$menuInner.off('scroll.createView').on('scroll.createView', function (e, updateValue) {
				if (!that.noScroll) scroll(this.scrollTop, updateValue);
				that.noScroll = false;
			});

			function scroll (scrollTop, init) {
				var size = that.selectpicker.current.elements.length,
						chunks = [],
						chunkSize,
						chunkCount,
						firstChunk,
						lastChunk,
						currentChunk,
						prevPositions,
						positionIsDifferent,
						previousElements,
						menuIsDifferent = true,
						isVirtual = that.isVirtual();

				that.selectpicker.view.scrollTop = scrollTop;

				if (isVirtual === true) {
					// if an option that is encountered that is wider than the current menu width, update the menu width accordingly
					if (that.sizeInfo.hasScrollBar && that.$menu[0].offsetWidth > that.sizeInfo.totalMenuWidth) {
						that.sizeInfo.menuWidth = that.$menu[0].offsetWidth;
						that.sizeInfo.totalMenuWidth = that.sizeInfo.menuWidth + that.sizeInfo.scrollBarWidth;
						that.$menu.css('min-width', that.sizeInfo.menuWidth);
					}
				}

				chunkSize = Math.ceil(that.sizeInfo.menuInnerHeight / that.sizeInfo.liHeight * 1.5); // number of options in a chunk
				chunkCount = Math.round(size / chunkSize) || 1; // number of chunks

				for (var i = 0; i < chunkCount; i++) {
					var endOfChunk = (i + 1) * chunkSize;

					if (i === chunkCount - 1) {
						endOfChunk = size;
					}

					chunks[i] = [
						(i) * chunkSize + (!i ? 0 : 1),
						endOfChunk
					];

					if (!size) break;

					if (currentChunk === undefined && scrollTop <= that.selectpicker.current.data[endOfChunk - 1].position - that.sizeInfo.menuInnerHeight) {
						currentChunk = i;
					}
				}

				if (currentChunk === undefined) currentChunk = 0;

				prevPositions = [that.selectpicker.view.position0, that.selectpicker.view.position1];

				// always display previous, current, and next chunks
				firstChunk = Math.max(0, currentChunk - 1);
				lastChunk = Math.min(chunkCount - 1, currentChunk + 1);

				that.selectpicker.view.position0 = isVirtual === false ? 0 : (Math.max(0, chunks[firstChunk][0]) || 0);
				that.selectpicker.view.position1 = isVirtual === false ? size : (Math.min(size, chunks[lastChunk][1]) || 0);

				positionIsDifferent = prevPositions[0] !== that.selectpicker.view.position0 || prevPositions[1] !== that.selectpicker.view.position1;

				if (that.activeIndex !== undefined) {
					prevActive = that.selectpicker.main.elements[that.prevActiveIndex];
					active = that.selectpicker.main.elements[that.activeIndex];
					selected = that.selectpicker.main.elements[that.selectedIndex];

					if (init) {
						if (that.activeIndex !== that.selectedIndex && active && active.length) {
							active.classList.remove('active');
							if (active.firstChild) active.firstChild.classList.remove('active');
						}
						that.activeIndex = undefined;
					}

					if (that.activeIndex && that.activeIndex !== that.selectedIndex && selected && selected.length) {
						selected.classList.remove('active');
						if (selected.firstChild) selected.firstChild.classList.remove('active');
					}
				}

				if (that.prevActiveIndex !== undefined && that.prevActiveIndex !== that.activeIndex && that.prevActiveIndex !== that.selectedIndex && prevActive && prevActive.length) {
					prevActive.classList.remove('active');
					if (prevActive.firstChild) prevActive.firstChild.classList.remove('active');
				}

				if (init || positionIsDifferent) {
					previousElements = that.selectpicker.view.visibleElements ? that.selectpicker.view.visibleElements.slice() : [];

					if (isVirtual === false) {
						that.selectpicker.view.visibleElements = that.selectpicker.current.elements;
					} else {
						that.selectpicker.view.visibleElements = that.selectpicker.current.elements.slice(that.selectpicker.view.position0, that.selectpicker.view.position1);
					}

					that.setOptionStatus();

					// if searching, check to make sure the list has actually been updated before updating DOM
					// this prevents unnecessary repaints
					if (isSearching || (isVirtual === false && init)) menuIsDifferent = !isEqual(previousElements, that.selectpicker.view.visibleElements);

					// if virtual scroll is disabled and not searching,
					// menu should never need to be updated more than once
					if ((init || isVirtual === true) && menuIsDifferent) {
						var menuInner = that.$menuInner[0],
								menuFragment = document.createDocumentFragment(),
								emptyMenu = menuInner.firstChild.cloneNode(false),
								marginTop,
								marginBottom,
								elements = that.selectpicker.view.visibleElements,
								toSanitize = [];

						// replace the existing UL with an empty one - this is faster than $.empty()
						menuInner.replaceChild(emptyMenu, menuInner.firstChild);

						for (var i = 0, visibleElementsLen = elements.length; i < visibleElementsLen; i++) {
							var element = elements[i],
									elText,
									elementData;

							if (that.options.sanitize) {
								elText = element.lastChild;

								if (elText) {
									elementData = that.selectpicker.current.data[i + that.selectpicker.view.position0];

									if (elementData && elementData.content && !elementData.sanitized) {
										toSanitize.push(elText);
										elementData.sanitized = true;
									}
								}
							}

							menuFragment.appendChild(element);
						}

						if (that.options.sanitize && toSanitize.length) {
							sanitizeHtml(toSanitize, that.options.whiteList, that.options.sanitizeFn);
						}

						if (isVirtual === true) {
							marginTop = (that.selectpicker.view.position0 === 0 ? 0 : that.selectpicker.current.data[that.selectpicker.view.position0 - 1].position);
							marginBottom = (that.selectpicker.view.position1 > size - 1 ? 0 : that.selectpicker.current.data[size - 1].position - that.selectpicker.current.data[that.selectpicker.view.position1 - 1].position);

							menuInner.firstChild.style.marginTop = marginTop + 'px';
							menuInner.firstChild.style.marginBottom = marginBottom + 'px';
						}

						menuInner.firstChild.appendChild(menuFragment);
					}
				}

				that.prevActiveIndex = that.activeIndex;

				if (!that.options.liveSearch) {
					that.$menuInner.trigger('focus');
				} else if (isSearching && init) {
					var index = 0,
							newActive;

					if (!that.selectpicker.view.canHighlight[index]) {
						index = 1 + that.selectpicker.view.canHighlight.slice(1).indexOf(true);
					}

					newActive = that.selectpicker.view.visibleElements[index];

					if (that.selectpicker.view.currentActive) {
						that.selectpicker.view.currentActive.classList.remove('active');
						if (that.selectpicker.view.currentActive.firstChild) that.selectpicker.view.currentActive.firstChild.classList.remove('active');
					}

					if (newActive) {
						newActive.classList.add('active');
						if (newActive.firstChild) newActive.firstChild.classList.add('active');
					}

					that.activeIndex = (that.selectpicker.current.data[index] || {}).index;
				}
			}

			$(window)
				.off('resize' + EVENT_KEY + '.' + this.selectId + '.createView')
				.on('resize' + EVENT_KEY + '.' + this.selectId + '.createView', function () {
					var isActive = that.$newElement.hasClass(classNames.SHOW);

					if (isActive) scroll(that.$menuInner[0].scrollTop);
				});
		},

		setPlaceholder: function () {
			var updateIndex = false;

			if (this.options.title && !this.multiple) {
				if (!this.selectpicker.view.titleOption) this.selectpicker.view.titleOption = document.createElement('option');

				// this option doesn't create a new <li> element, but does add a new option at the start,
				// so startIndex should increase to prevent having to check every option for the bs-title-option class
				updateIndex = true;

				var element = this.$element[0],
						isSelected = false,
						titleNotAppended = !this.selectpicker.view.titleOption.parentNode;

				if (titleNotAppended) {
					// Use native JS to prepend option (faster)
					this.selectpicker.view.titleOption.className = 'bs-title-option';
					this.selectpicker.view.titleOption.value = '';

					// Check if selected or data-selected attribute is already set on an option. If not, select the titleOption option.
					// the selected item may have been changed by user or programmatically before the bootstrap select plugin runs,
					// if so, the select will have the data-selected attribute
					var $opt = $(element.options[element.selectedIndex]);
					isSelected = $opt.attr('selected') === undefined && this.$element.data('selected') === undefined;
				}

				if (titleNotAppended || this.selectpicker.view.titleOption.index !== 0) {
					element.insertBefore(this.selectpicker.view.titleOption, element.firstChild);
				}

				// Set selected *after* appending to select,
				// otherwise the option doesn't get selected in IE
				// set using selectedIndex, as setting the selected attr to true here doesn't work in IE11
				if (isSelected) element.selectedIndex = 0;
			}

			return updateIndex;
		},

		createLi: function () {
			var that = this,
					iconBase = this.options.iconBase,
					optionSelector = ':not([hidden]):not([data-hidden="true"])',
					mainElements = [],
					mainData = [],
					widestOptionLength = 0,
					optID = 0,
					startIndex = this.setPlaceholder() ? 1 : 0; // append the titleOption if necessary and skip the first option in the loop

			if (this.options.hideDisabled) optionSelector += ':not(:disabled)';

			if ((that.options.showTick || that.multiple) && !elementTemplates.checkMark.parentNode) {
				elementTemplates.checkMark.className = iconBase + ' ' + that.options.tickIcon + ' check-mark';
				elementTemplates.a.appendChild(elementTemplates.checkMark);
			}

			var selectOptions = this.$element[0].querySelectorAll('select > *' + optionSelector);

			function addDivider (config) {
				var previousData = mainData[mainData.length - 1];

				// ensure optgroup doesn't create back-to-back dividers
				if (
					previousData &&
					previousData.type === 'divider' &&
					(previousData.optID || config.optID)
				) {
					return;
				}

				config = config || {};
				config.type = 'divider';

				mainElements.push(
					generateOption.li(
						false,
						classNames.DIVIDER,
						(config.optID ? config.optID + 'div' : undefined)
					)
				);

				mainData.push(config);
			}

			function addOption (option, config) {
				config = config || {};

				config.divider = option.getAttribute('data-divider') === 'true';

				if (config.divider) {
					addDivider({
						optID: config.optID
					});
				} else {
					var liIndex = mainData.length,
							cssText = option.style.cssText,
							inlineStyle = cssText ? htmlEscape(cssText) : '',
							optionClass = (option.className || '') + (config.optgroupClass || '');

					if (config.optID) optionClass = 'opt ' + optionClass;

					config.text = option.textContent;

					config.content = option.getAttribute('data-content');
					config.tokens = option.getAttribute('data-tokens');
					config.subtext = option.getAttribute('data-subtext');
					config.icon = option.getAttribute('data-icon');
					config.iconBase = iconBase;

					var textElement = generateOption.text(config);

					mainElements.push(
						generateOption.li(
							generateOption.a(
								textElement,
								optionClass,
								inlineStyle
							),
							'',
							config.optID
						)
					);

					option.liIndex = liIndex;

					config.display = config.content || config.text;
					config.type = 'option';
					config.index = liIndex;
					config.option = option;
					config.disabled = config.disabled || option.disabled;

					mainData.push(config);

					var combinedLength = 0;

					// count the number of characters in the option - not perfect, but should work in most cases
					if (config.display) combinedLength += config.display.length;
					if (config.subtext) combinedLength += config.subtext.length;
					// if there is an icon, ensure this option's width is checked
					if (config.icon) combinedLength += 1;

					if (combinedLength > widestOptionLength) {
						widestOptionLength = combinedLength;

						// guess which option is the widest
						// use this when calculating menu width
						// not perfect, but it's fast, and the width will be updating accordingly when scrolling
						that.selectpicker.view.widestOption = mainElements[mainElements.length - 1];
					}
				}
			}

			function addOptgroup (index, selectOptions) {
				var optgroup = selectOptions[index],
						previous = selectOptions[index - 1],
						next = selectOptions[index + 1],
						options = optgroup.querySelectorAll('option' + optionSelector);

				if (!options.length) return;

				var config = {
							label: htmlEscape(optgroup.label),
							subtext: optgroup.getAttribute('data-subtext'),
							icon: optgroup.getAttribute('data-icon'),
							iconBase: iconBase
						},
						optgroupClass = ' ' + (optgroup.className || ''),
						headerIndex,
						lastIndex;

				optID++;

				if (previous) {
					addDivider({ optID: optID });
				}

				var labelElement = generateOption.label(config);

				mainElements.push(
					generateOption.li(labelElement, 'dropdown-header' + optgroupClass, optID)
				);

				mainData.push({
					display: config.label,
					subtext: config.subtext,
					type: 'optgroup-label',
					optID: optID
				});

				for (var j = 0, len = options.length; j < len; j++) {
					var option = options[j];

					if (j === 0) {
						headerIndex = mainData.length - 1;
						lastIndex = headerIndex + len;
					}

					addOption(option, {
						headerIndex: headerIndex,
						lastIndex: lastIndex,
						optID: optID,
						optgroupClass: optgroupClass,
						disabled: optgroup.disabled
					});
				}

				if (next) {
					addDivider({ optID: optID });
				}
			}

			for (var len = selectOptions.length; startIndex < len; startIndex++) {
				var item = selectOptions[startIndex];

				if (item.tagName !== 'OPTGROUP') {
					addOption(item, {});
				} else {
					addOptgroup(startIndex, selectOptions);
				}
			}

			this.selectpicker.main.elements = mainElements;
			this.selectpicker.main.data = mainData;

			this.selectpicker.current = this.selectpicker.main;
		},

		findLis: function () {
			return this.$menuInner.find('.inner > li');
		},

		render: function () {
			// ensure titleOption is appended and selected (if necessary) before getting selectedOptions
			this.setPlaceholder();

			var that = this,
					selectedOptions = this.$element[0].selectedOptions,
					selectedCount = selectedOptions.length,
					button = this.$button[0],
					buttonInner = button.querySelector('.filter-option-inner-inner'),
					multipleSeparator = document.createTextNode(this.options.multipleSeparator),
					titleFragment = elementTemplates.fragment.cloneNode(false),
					showCount,
					countMax,
					hasContent = false;

			this.togglePlaceholder();

			this.tabIndex();

			if (this.options.selectedTextFormat === 'static') {
				titleFragment = generateOption.text({ text: this.options.title }, true);
			} else {
				showCount = this.multiple && this.options.selectedTextFormat.indexOf('count') !== -1 && selectedCount > 1;

				// determine if the number of selected options will be shown (showCount === true)
				if (showCount) {
					countMax = this.options.selectedTextFormat.split('>');
					showCount = (countMax.length > 1 && selectedCount > countMax[1]) || (countMax.length === 1 && selectedCount >= 2);
				}

				// only loop through all selected options if the count won't be shown
				if (showCount === false) {
					for (var selectedIndex = 0; selectedIndex < selectedCount; selectedIndex++) {
						if (selectedIndex < 50) {
							var option = selectedOptions[selectedIndex],
									titleOptions = {},
									thisData = {
										content: option.getAttribute('data-content'),
										subtext: option.getAttribute('data-subtext'),
										icon: option.getAttribute('data-icon')
									};

							if (this.multiple && selectedIndex > 0) {
								titleFragment.appendChild(multipleSeparator.cloneNode(false));
							}

							if (option.title) {
								titleOptions.text = option.title;
							} else if (thisData.content && that.options.showContent) {
								titleOptions.content = thisData.content.toString();
								hasContent = true;
							} else {
								if (that.options.showIcon) {
									titleOptions.icon = thisData.icon;
									titleOptions.iconBase = this.options.iconBase;
								}
								if (that.options.showSubtext && !that.multiple && thisData.subtext) titleOptions.subtext = ' ' + thisData.subtext;
								titleOptions.text = option.textContent.trim();
							}

							titleFragment.appendChild(generateOption.text(titleOptions, true));
						} else {
							break;
						}
					}

					// add ellipsis
					if (selectedCount > 49) {
						titleFragment.appendChild(document.createTextNode('...'));
					}
				} else {
					var optionSelector = ':not([hidden]):not([data-hidden="true"]):not([data-divider="true"])';
					if (this.options.hideDisabled) optionSelector += ':not(:disabled)';

					// If this is a multiselect, and selectedTextFormat is count, then show 1 of 2 selected, etc.
					var totalCount = this.$element[0].querySelectorAll('select > option' + optionSelector + ', optgroup' + optionSelector + ' option' + optionSelector).length,
							tr8nText = (typeof this.options.countSelectedText === 'function') ? this.options.countSelectedText(selectedCount, totalCount) : this.options.countSelectedText;

					titleFragment = generateOption.text({
						text: tr8nText.replace('{0}', selectedCount.toString()).replace('{1}', totalCount.toString())
					}, true);
				}
			}

			if (this.options.title == undefined) {
				// use .attr to ensure undefined is returned if title attribute is not set
				this.options.title = this.$element.attr('title');
			}

			// If the select doesn't have a title, then use the default, or if nothing is set at all, use noneSelectedText
			if (!titleFragment.childNodes.length) {
				titleFragment = generateOption.text({
					text: typeof this.options.title !== 'undefined' ? this.options.title : this.options.noneSelectedText
				}, true);
			}

			// strip all HTML tags and trim the result, then unescape any escaped tags
			button.title = titleFragment.textContent.replace(/<[^>]*>?/g, '').trim();

			if (this.options.sanitize && hasContent) {
				sanitizeHtml([titleFragment], that.options.whiteList, that.options.sanitizeFn);
			}

			buttonInner.innerHTML = '';
			buttonInner.appendChild(titleFragment);

			if (version.major < 4 && this.$newElement[0].classList.contains('bs3-has-addon')) {
				var filterExpand = button.querySelector('.filter-expand'),
						clone = buttonInner.cloneNode(true);

				clone.className = 'filter-expand';

				if (filterExpand) {
					button.replaceChild(clone, filterExpand);
				} else {
					button.appendChild(clone);
				}
			}

			this.$element.trigger('rendered' + EVENT_KEY);
		},

		/**
		 * @param [style]
		 * @param [status]
		 */
		setStyle: function (newStyle, status) {
			var button = this.$button[0],
					newElement = this.$newElement[0],
					style = this.options.style.trim(),
					buttonClass;

			if (this.$element.attr('class')) {
				this.$newElement.addClass(this.$element.attr('class').replace(/selectpicker|mobile-device|bs-select-hidden|validate\[.*\]/gi, ''));
			}

			if (version.major < 4) {
				newElement.classList.add('bs3');

				if (newElement.parentNode.classList.contains('input-group') &&
						(newElement.previousElementSibling || newElement.nextElementSibling) &&
						(newElement.previousElementSibling || newElement.nextElementSibling).classList.contains('input-group-addon')
				) {
					newElement.classList.add('bs3-has-addon');
				}
			}

			if (newStyle) {
				buttonClass = newStyle.trim();
			} else {
				buttonClass = style;
			}

			if (status == 'add') {
				if (buttonClass) button.classList.add.apply(button.classList, buttonClass.split(' '));
			} else if (status == 'remove') {
				if (buttonClass) button.classList.remove.apply(button.classList, buttonClass.split(' '));
			} else {
				if (style) button.classList.remove.apply(button.classList, style.split(' '));
				if (buttonClass) button.classList.add.apply(button.classList, buttonClass.split(' '));
			}
		},

		liHeight: function (refresh) {
			if (!refresh && (this.options.size === false || this.sizeInfo)) return;

			if (!this.sizeInfo) this.sizeInfo = {};

			var newElement = document.createElement('div'),
					menu = document.createElement('div'),
					menuInner = document.createElement('div'),
					menuInnerInner = document.createElement('ul'),
					divider = document.createElement('li'),
					dropdownHeader = document.createElement('li'),
					li = document.createElement('li'),
					a = document.createElement('a'),
					text = document.createElement('span'),
					header = this.options.header && this.$menu.find('.' + classNames.POPOVERHEADER).length > 0 ? this.$menu.find('.' + classNames.POPOVERHEADER)[0].cloneNode(true) : null,
					search = this.options.liveSearch ? document.createElement('div') : null,
					actions = this.options.actionsBox && this.multiple && this.$menu.find('.bs-actionsbox').length > 0 ? this.$menu.find('.bs-actionsbox')[0].cloneNode(true) : null,
					doneButton = this.options.doneButton && this.multiple && this.$menu.find('.bs-donebutton').length > 0 ? this.$menu.find('.bs-donebutton')[0].cloneNode(true) : null,
					firstOption = this.$element.find('option')[0];

			this.sizeInfo.selectWidth = this.$newElement[0].offsetWidth;

			text.className = 'text';
			a.className = 'dropdown-item ' + (firstOption ? firstOption.className : '');
			newElement.className = this.$menu[0].parentNode.className + ' ' + classNames.SHOW;
			newElement.style.width = this.sizeInfo.selectWidth + 'px';
			if (this.options.width === 'auto') menu.style.minWidth = 0;
			menu.className = classNames.MENU + ' ' + classNames.SHOW;
			menuInner.className = 'inner ' + classNames.SHOW;
			menuInnerInner.className = classNames.MENU + ' inner ' + (version.major === '4' ? classNames.SHOW : '');
			divider.className = classNames.DIVIDER;
			dropdownHeader.className = 'dropdown-header';

			text.appendChild(document.createTextNode('\u200b'));
			a.appendChild(text);
			li.appendChild(a);
			dropdownHeader.appendChild(text.cloneNode(true));

			if (this.selectpicker.view.widestOption) {
				menuInnerInner.appendChild(this.selectpicker.view.widestOption.cloneNode(true));
			}

			menuInnerInner.appendChild(li);
			menuInnerInner.appendChild(divider);
			menuInnerInner.appendChild(dropdownHeader);
			if (header) menu.appendChild(header);
			if (search) {
				var input = document.createElement('input');
				search.className = 'bs-searchbox';
				input.className = 'form-control';
				search.appendChild(input);
				menu.appendChild(search);
			}
			if (actions) menu.appendChild(actions);
			menuInner.appendChild(menuInnerInner);
			menu.appendChild(menuInner);
			if (doneButton) menu.appendChild(doneButton);
			newElement.appendChild(menu);

			document.body.appendChild(newElement);

			var liHeight = li.offsetHeight,
					dropdownHeaderHeight = dropdownHeader ? dropdownHeader.offsetHeight : 0,
					headerHeight = header ? header.offsetHeight : 0,
					searchHeight = search ? search.offsetHeight : 0,
					actionsHeight = actions ? actions.offsetHeight : 0,
					doneButtonHeight = doneButton ? doneButton.offsetHeight : 0,
					dividerHeight = $(divider).outerHeight(true),
					// fall back to jQuery if getComputedStyle is not supported
					menuStyle = window.getComputedStyle ? window.getComputedStyle(menu) : false,
					menuWidth = menu.offsetWidth,
					$menu = menuStyle ? null : $(menu),
					menuPadding = {
						vert: toInteger(menuStyle ? menuStyle.paddingTop : $menu.css('paddingTop')) +
									toInteger(menuStyle ? menuStyle.paddingBottom : $menu.css('paddingBottom')) +
									toInteger(menuStyle ? menuStyle.borderTopWidth : $menu.css('borderTopWidth')) +
									toInteger(menuStyle ? menuStyle.borderBottomWidth : $menu.css('borderBottomWidth')),
						horiz: toInteger(menuStyle ? menuStyle.paddingLeft : $menu.css('paddingLeft')) +
									toInteger(menuStyle ? menuStyle.paddingRight : $menu.css('paddingRight')) +
									toInteger(menuStyle ? menuStyle.borderLeftWidth : $menu.css('borderLeftWidth')) +
									toInteger(menuStyle ? menuStyle.borderRightWidth : $menu.css('borderRightWidth'))
					},
					menuExtras = {
						vert: menuPadding.vert +
									toInteger(menuStyle ? menuStyle.marginTop : $menu.css('marginTop')) +
									toInteger(menuStyle ? menuStyle.marginBottom : $menu.css('marginBottom')) + 2,
						horiz: menuPadding.horiz +
									toInteger(menuStyle ? menuStyle.marginLeft : $menu.css('marginLeft')) +
									toInteger(menuStyle ? menuStyle.marginRight : $menu.css('marginRight')) + 2
					},
					scrollBarWidth;

			menuInner.style.overflowY = 'scroll';

			scrollBarWidth = menu.offsetWidth - menuWidth;

			document.body.removeChild(newElement);

			this.sizeInfo.liHeight = liHeight;
			this.sizeInfo.dropdownHeaderHeight = dropdownHeaderHeight;
			this.sizeInfo.headerHeight = headerHeight;
			this.sizeInfo.searchHeight = searchHeight;
			this.sizeInfo.actionsHeight = actionsHeight;
			this.sizeInfo.doneButtonHeight = doneButtonHeight;
			this.sizeInfo.dividerHeight = dividerHeight;
			this.sizeInfo.menuPadding = menuPadding;
			this.sizeInfo.menuExtras = menuExtras;
			this.sizeInfo.menuWidth = menuWidth;
			this.sizeInfo.totalMenuWidth = this.sizeInfo.menuWidth;
			this.sizeInfo.scrollBarWidth = scrollBarWidth;
			this.sizeInfo.selectHeight = this.$newElement[0].offsetHeight;

			this.setPositionData();
		},

		getSelectPosition: function () {
			var that = this,
					$window = $(window),
					pos = that.$newElement.offset(),
					$container = $(that.options.container),
					containerPos;

			if (that.options.container && $container.length && !$container.is('body')) {
				containerPos = $container.offset();
				containerPos.top += parseInt($container.css('borderTopWidth'));
				containerPos.left += parseInt($container.css('borderLeftWidth'));
			} else {
				containerPos = { top: 0, left: 0 };
			}

			var winPad = that.options.windowPadding;

			this.sizeInfo.selectOffsetTop = pos.top - containerPos.top - $window.scrollTop();
			this.sizeInfo.selectOffsetBot = $window.height() - this.sizeInfo.selectOffsetTop - this.sizeInfo.selectHeight - containerPos.top - winPad[2];
			this.sizeInfo.selectOffsetLeft = pos.left - containerPos.left - $window.scrollLeft();
			this.sizeInfo.selectOffsetRight = $window.width() - this.sizeInfo.selectOffsetLeft - this.sizeInfo.selectWidth - containerPos.left - winPad[1];
			this.sizeInfo.selectOffsetTop -= winPad[0];
			this.sizeInfo.selectOffsetLeft -= winPad[3];
		},

		setMenuSize: function (isAuto) {
			this.getSelectPosition();

			var selectWidth = this.sizeInfo.selectWidth,
					liHeight = this.sizeInfo.liHeight,
					headerHeight = this.sizeInfo.headerHeight,
					searchHeight = this.sizeInfo.searchHeight,
					actionsHeight = this.sizeInfo.actionsHeight,
					doneButtonHeight = this.sizeInfo.doneButtonHeight,
					divHeight = this.sizeInfo.dividerHeight,
					menuPadding = this.sizeInfo.menuPadding,
					menuInnerHeight,
					menuHeight,
					divLength = 0,
					minHeight,
					_minHeight,
					maxHeight,
					menuInnerMinHeight,
					estimate;

			if (this.options.dropupAuto) {
				// Get the estimated height of the menu without scrollbars.
				// This is useful for smaller menus, where there might be plenty of room
				// below the button without setting dropup, but we can't know
				// the exact height of the menu until createView is called later
				estimate = liHeight * this.selectpicker.current.elements.length + menuPadding.vert;
				this.$newElement.toggleClass(classNames.DROPUP, this.sizeInfo.selectOffsetTop - this.sizeInfo.selectOffsetBot > this.sizeInfo.menuExtras.vert && estimate + this.sizeInfo.menuExtras.vert + 50 > this.sizeInfo.selectOffsetBot);
			}

			if (this.options.size === 'auto') {
				_minHeight = this.selectpicker.current.elements.length > 3 ? this.sizeInfo.liHeight * 3 + this.sizeInfo.menuExtras.vert - 2 : 0;
				menuHeight = this.sizeInfo.selectOffsetBot - this.sizeInfo.menuExtras.vert;
				minHeight = _minHeight + headerHeight + searchHeight + actionsHeight + doneButtonHeight;
				menuInnerMinHeight = Math.max(_minHeight - menuPadding.vert, 0);

				if (this.$newElement.hasClass(classNames.DROPUP)) {
					menuHeight = this.sizeInfo.selectOffsetTop - this.sizeInfo.menuExtras.vert;
				}

				maxHeight = menuHeight;
				menuInnerHeight = menuHeight - headerHeight - searchHeight - actionsHeight - doneButtonHeight - menuPadding.vert;
			} else if (this.options.size && this.options.size != 'auto' && this.selectpicker.current.elements.length > this.options.size) {
				for (var i = 0; i < this.options.size; i++) {
					if (this.selectpicker.current.data[i].type === 'divider') divLength++;
				}

				menuHeight = liHeight * this.options.size + divLength * divHeight + menuPadding.vert;
				menuInnerHeight = menuHeight - menuPadding.vert;
				maxHeight = menuHeight + headerHeight + searchHeight + actionsHeight + doneButtonHeight;
				minHeight = menuInnerMinHeight = '';
			}

			if (this.options.dropdownAlignRight === 'auto') {
				this.$menu.toggleClass(classNames.MENURIGHT, this.sizeInfo.selectOffsetLeft > this.sizeInfo.selectOffsetRight && this.sizeInfo.selectOffsetRight < (this.sizeInfo.totalMenuWidth - selectWidth));
			}

			this.$menu.css({
				'max-height': maxHeight + 'px',
				'overflow': 'hidden',
				'min-height': minHeight + 'px'
			});

			this.$menuInner.css({
				'max-height': menuInnerHeight + 'px',
				'overflow-y': 'auto',
				'min-height': menuInnerMinHeight + 'px'
			});

			// ensure menuInnerHeight is always a positive number to prevent issues calculating chunkSize in createView
			this.sizeInfo.menuInnerHeight = Math.max(menuInnerHeight, 1);

			if (this.selectpicker.current.data.length && this.selectpicker.current.data[this.selectpicker.current.data.length - 1].position > this.sizeInfo.menuInnerHeight) {
				this.sizeInfo.hasScrollBar = true;
				this.sizeInfo.totalMenuWidth = this.sizeInfo.menuWidth + this.sizeInfo.scrollBarWidth;

				this.$menu.css('min-width', this.sizeInfo.totalMenuWidth);
			}

			if (this.dropdown && this.dropdown._popper) this.dropdown._popper.update();
		},

		setSize: function (refresh) {
			this.liHeight(refresh);

			if (this.options.header) this.$menu.css('padding-top', 0);
			if (this.options.size === false) return;

			var that = this,
					$window = $(window),
					selectedIndex,
					offset = 0;

			this.setMenuSize();

			if (this.options.liveSearch) {
				this.$searchbox
					.off('input.setMenuSize propertychange.setMenuSize')
					.on('input.setMenuSize propertychange.setMenuSize', function () {
						return that.setMenuSize();
					});
			}

			if (this.options.size === 'auto') {
				$window
					.off('resize' + EVENT_KEY + '.' + this.selectId + '.setMenuSize' + ' scroll' + EVENT_KEY + '.' + this.selectId + '.setMenuSize')
					.on('resize' + EVENT_KEY + '.' + this.selectId + '.setMenuSize' + ' scroll' + EVENT_KEY + '.' + this.selectId + '.setMenuSize', function () {
						return that.setMenuSize();
					});
			} else if (this.options.size && this.options.size != 'auto' && this.selectpicker.current.elements.length > this.options.size) {
				$window.off('resize' + EVENT_KEY + '.' + this.selectId + '.setMenuSize' + ' scroll' + EVENT_KEY + '.' + this.selectId + '.setMenuSize');
			}

			if (refresh) {
				offset = this.$menuInner[0].scrollTop;
			} else if (!that.multiple) {
				var element = that.$element[0];
				selectedIndex = (element.options[element.selectedIndex] || {}).liIndex;

				if (typeof selectedIndex === 'number' && that.options.size !== false) {
					offset = that.sizeInfo.liHeight * selectedIndex;
					offset = offset - (that.sizeInfo.menuInnerHeight / 2) + (that.sizeInfo.liHeight / 2);
				}
			}

			that.createView(false, offset);
		},

		setWidth: function () {
			var that = this;

			if (this.options.width === 'auto') {
				requestAnimationFrame(function () {
					that.$menu.css('min-width', '0');

					that.$element.on('loaded' + EVENT_KEY, function () {
						that.liHeight();
						that.setMenuSize();

						// Get correct width if element is hidden
						var $selectClone = that.$newElement.clone().appendTo('body'),
								btnWidth = $selectClone.css('width', 'auto').children('button').outerWidth();

						$selectClone.remove();

						// Set width to whatever's larger, button title or longest option
						that.sizeInfo.selectWidth = Math.max(that.sizeInfo.totalMenuWidth, btnWidth);
						that.$newElement.css('width', that.sizeInfo.selectWidth + 'px');
					});
				});
			} else if (this.options.width === 'fit') {
				// Remove inline min-width so width can be changed from 'auto'
				this.$menu.css('min-width', '');
				this.$newElement.css('width', '').addClass('fit-width');
			} else if (this.options.width) {
				// Remove inline min-width so width can be changed from 'auto'
				this.$menu.css('min-width', '');
				this.$newElement.css('width', this.options.width);
			} else {
				// Remove inline min-width/width so width can be changed
				this.$menu.css('min-width', '');
				this.$newElement.css('width', '');
			}
			// Remove fit-width class if width is changed programmatically
			if (this.$newElement.hasClass('fit-width') && this.options.width !== 'fit') {
				this.$newElement[0].classList.remove('fit-width');
			}
		},

		selectPosition: function () {
			this.$bsContainer = $('<div class="bs-container" />');

			var that = this,
					$container = $(this.options.container),
					pos,
					containerPos,
					actualHeight,
					getPlacement = function ($element) {
						var containerPosition = {},
								// fall back to dropdown's default display setting if display is not manually set
								display = that.options.display || (
									// Bootstrap 3 doesn't have $.fn.dropdown.Constructor.Default
									$.fn.dropdown.Constructor.Default ? $.fn.dropdown.Constructor.Default.display
									: false
								);

						that.$bsContainer.addClass($element.attr('class').replace(/form-control|fit-width/gi, '')).toggleClass(classNames.DROPUP, $element.hasClass(classNames.DROPUP));
						pos = $element.offset();

						if (!$container.is('body')) {
							containerPos = $container.offset();
							containerPos.top += parseInt($container.css('borderTopWidth')) - $container.scrollTop();
							containerPos.left += parseInt($container.css('borderLeftWidth')) - $container.scrollLeft();
						} else {
							containerPos = { top: 0, left: 0 };
						}

						actualHeight = $element.hasClass(classNames.DROPUP) ? 0 : $element[0].offsetHeight;

						// Bootstrap 4+ uses Popper for menu positioning
						if (version.major < 4 || display === 'static') {
							containerPosition.top = pos.top - containerPos.top + actualHeight;
							containerPosition.left = pos.left - containerPos.left;
						}

						containerPosition.width = $element[0].offsetWidth;

						that.$bsContainer.css(containerPosition);
					};

			this.$button.on('click.bs.dropdown.data-api', function () {
				if (that.isDisabled()) {
					return;
				}

				getPlacement(that.$newElement);

				that.$bsContainer
					.appendTo(that.options.container)
					.toggleClass(classNames.SHOW, !that.$button.hasClass(classNames.SHOW))
					.append(that.$menu);
			});

			$(window)
				.off('resize' + EVENT_KEY + '.' + this.selectId + ' scroll' + EVENT_KEY + '.' + this.selectId)
				.on('resize' + EVENT_KEY + '.' + this.selectId + ' scroll' + EVENT_KEY + '.' + this.selectId, function () {
					var isActive = that.$newElement.hasClass(classNames.SHOW);

					if (isActive) getPlacement(that.$newElement);
				});

			this.$element.on('hide' + EVENT_KEY, function () {
				that.$menu.data('height', that.$menu.height());
				that.$bsContainer.detach();
			});
		},

		setOptionStatus: function () {
			var that = this;

			that.noScroll = false;

			if (that.selectpicker.view.visibleElements && that.selectpicker.view.visibleElements.length) {
				for (var i = 0; i < that.selectpicker.view.visibleElements.length; i++) {
					var liData = that.selectpicker.current.data[i + that.selectpicker.view.position0],
							option = liData.option;

					if (option) {
						that.setDisabled(
							liData.index,
							liData.disabled
						);

						that.setSelected(
							liData.index,
							option.selected
						);
					}
				}
			}
		},

		/**
		 * @param {number} index - the index of the option that is being changed
		 * @param {boolean} selected - true if the option is being selected, false if being deselected
		 */
		setSelected: function (index, selected) {
			var li = this.selectpicker.main.elements[index],
					liData = this.selectpicker.main.data[index],
					activeIndexIsSet = this.activeIndex !== undefined,
					thisIsActive = this.activeIndex === index,
					prevActive,
					a,
					// if current option is already active
					// OR
					// if the current option is being selected, it's NOT multiple, and
					// activeIndex is undefined:
					//  - when the menu is first being opened, OR
					//  - after a search has been performed, OR
					//  - when retainActive is false when selecting a new option (i.e. index of the newly selected option is not the same as the current activeIndex)
					keepActive = thisIsActive || (selected && !this.multiple && !activeIndexIsSet);

			liData.selected = selected;

			a = li.firstChild;

			if (selected) {
				this.selectedIndex = index;
			}

			li.classList.toggle('selected', selected);
			li.classList.toggle('active', keepActive);

			if (keepActive) {
				this.selectpicker.view.currentActive = li;
				this.activeIndex = index;
			}

			if (a) {
				a.classList.toggle('selected', selected);
				a.classList.toggle('active', keepActive);
				a.setAttribute('aria-selected', selected);
			}

			if (!keepActive) {
				if (!activeIndexIsSet && selected && this.prevActiveIndex !== undefined) {
					prevActive = this.selectpicker.main.elements[this.prevActiveIndex];

					prevActive.classList.remove('active');
					if (prevActive.firstChild) {
						prevActive.firstChild.classList.remove('active');
					}
				}
			}
		},

		/**
		 * @param {number} index - the index of the option that is being disabled
		 * @param {boolean} disabled - true if the option is being disabled, false if being enabled
		 */
		setDisabled: function (index, disabled) {
			var li = this.selectpicker.main.elements[index],
					a;

			this.selectpicker.main.data[index].disabled = disabled;

			a = li.firstChild;

			li.classList.toggle(classNames.DISABLED, disabled);

			if (a) {
				if (version.major === '4') a.classList.toggle(classNames.DISABLED, disabled);

				a.setAttribute('aria-disabled', disabled);

				if (disabled) {
					a.setAttribute('tabindex', -1);
				} else {
					a.setAttribute('tabindex', 0);
				}
			}
		},

		isDisabled: function () {
			return this.$element[0].disabled;
		},

		checkDisabled: function () {
			var that = this;

			if (this.isDisabled()) {
				this.$newElement[0].classList.add(classNames.DISABLED);
				this.$button.addClass(classNames.DISABLED).attr('tabindex', -1).attr('aria-disabled', true);
			} else {
				if (this.$button[0].classList.contains(classNames.DISABLED)) {
					this.$newElement[0].classList.remove(classNames.DISABLED);
					this.$button.removeClass(classNames.DISABLED).attr('aria-disabled', false);
				}

				if (this.$button.attr('tabindex') == -1 && !this.$element.data('tabindex')) {
					this.$button.removeAttr('tabindex');
				}
			}

			this.$button.on('click', function () {
				return !that.isDisabled();
			});
		},

		togglePlaceholder: function () {
			// much faster than calling $.val()
			var element = this.$element[0],
					selectedIndex = element.selectedIndex,
					nothingSelected = selectedIndex === -1;

			if (!nothingSelected && !element.options[selectedIndex].value) nothingSelected = true;

			this.$button.toggleClass('bs-placeholder', nothingSelected);
		},

		tabIndex: function () {
			if (this.$element.data('tabindex') !== this.$element.attr('tabindex') &&
				(this.$element.attr('tabindex') !== -98 && this.$element.attr('tabindex') !== '-98')) {
				this.$element.data('tabindex', this.$element.attr('tabindex'));
				this.$button.attr('tabindex', this.$element.data('tabindex'));
			}

			this.$element.attr('tabindex', -98);
		},

		clickListener: function () {
			var that = this,
					$document = $(document);

			$document.data('spaceSelect', false);

			this.$button.on('keyup', function (e) {
				if (/(32)/.test(e.keyCode.toString(10)) && $document.data('spaceSelect')) {
					e.preventDefault();
					$document.data('spaceSelect', false);
				}
			});

			this.$newElement.on('show.bs.dropdown', function () {
				if (version.major > 3 && !that.dropdown) {
					that.dropdown = that.$button.data('bs.dropdown');
					that.dropdown._menu = that.$menu[0];
				}
			});

			this.$button.on('click.bs.dropdown.data-api', function () {
				if (!that.$newElement.hasClass(classNames.SHOW)) {
					that.setSize();
				}
			});

			function setFocus () {
				if (that.options.liveSearch) {
					that.$searchbox.trigger('focus');
				} else {
					that.$menuInner.trigger('focus');
				}
			}

			function checkPopperExists () {
				if (that.dropdown && that.dropdown._popper && that.dropdown._popper.state.isCreated) {
					setFocus();
				} else {
					requestAnimationFrame(checkPopperExists);
				}
			}

			this.$element.on('shown' + EVENT_KEY, function () {
				if (that.$menuInner[0].scrollTop !== that.selectpicker.view.scrollTop) {
					that.$menuInner[0].scrollTop = that.selectpicker.view.scrollTop;
				}

				if (version.major > 3) {
					requestAnimationFrame(checkPopperExists);
				} else {
					setFocus();
				}
			});

			this.$menuInner.on('click', 'li a', function (e, retainActive) {
				var $this = $(this),
						position0 = that.isVirtual() ? that.selectpicker.view.position0 : 0,
						clickedData = that.selectpicker.current.data[$this.parent().index() + position0],
						clickedIndex = clickedData.index,
						prevValue = getSelectValues(that.$element[0]),
						prevIndex = that.$element.prop('selectedIndex'),
						triggerChange = true;

				// Don't close on multi choice menu
				if (that.multiple && that.options.maxOptions !== 1) {
					e.stopPropagation();
				}

				e.preventDefault();

				// Don't run if the select is disabled
				if (!that.isDisabled() && !$this.parent().hasClass(classNames.DISABLED)) {
					var $options = that.$element.find('option'),
							option = clickedData.option,
							$option = $(option),
							state = option.selected,
							$optgroup = $option.parent('optgroup'),
							$optgroupOptions = $optgroup.find('option'),
							maxOptions = that.options.maxOptions,
							maxOptionsGrp = $optgroup.data('maxOptions') || false;

					if (clickedIndex === that.activeIndex) retainActive = true;

					if (!retainActive) {
						that.prevActiveIndex = that.activeIndex;
						that.activeIndex = undefined;
					}

					if (!that.multiple) { // Deselect all others if not multi select box
						$options.prop('selected', false);
						option.selected = true;
						that.setSelected(clickedIndex, true);
					} else { // Toggle the one we have chosen if we are multi select.
						option.selected = !state;

						that.setSelected(clickedIndex, !state);
						$this.trigger('blur');

						if (maxOptions !== false || maxOptionsGrp !== false) {
							var maxReached = maxOptions < $options.filter(':selected').length,
									maxReachedGrp = maxOptionsGrp < $optgroup.find('option:selected').length;

							if ((maxOptions && maxReached) || (maxOptionsGrp && maxReachedGrp)) {
								if (maxOptions && maxOptions == 1) {
									$options.prop('selected', false);
									$option.prop('selected', true);

									for (var i = 0; i < $options.length; i++) {
										that.setSelected(i, false);
									}

									that.setSelected(clickedIndex, true);
								} else if (maxOptionsGrp && maxOptionsGrp == 1) {
									$optgroup.find('option:selected').prop('selected', false);
									$option.prop('selected', true);

									for (var i = 0; i < $optgroupOptions.length; i++) {
										var option = $optgroupOptions[i];
										that.setSelected($options.index(option), false);
									}

									that.setSelected(clickedIndex, true);
								} else {
									var maxOptionsText = typeof that.options.maxOptionsText === 'string' ? [that.options.maxOptionsText, that.options.maxOptionsText] : that.options.maxOptionsText,
											maxOptionsArr = typeof maxOptionsText === 'function' ? maxOptionsText(maxOptions, maxOptionsGrp) : maxOptionsText,
											maxTxt = maxOptionsArr[0].replace('{n}', maxOptions),
											maxTxtGrp = maxOptionsArr[1].replace('{n}', maxOptionsGrp),
											$notify = $('<div class="notify"></div>');
									// If {var} is set in array, replace it
									/** @deprecated */
									if (maxOptionsArr[2]) {
										maxTxt = maxTxt.replace('{var}', maxOptionsArr[2][maxOptions > 1 ? 0 : 1]);
										maxTxtGrp = maxTxtGrp.replace('{var}', maxOptionsArr[2][maxOptionsGrp > 1 ? 0 : 1]);
									}

									$option.prop('selected', false);

									that.$menu.append($notify);

									if (maxOptions && maxReached) {
										$notify.append($('<div>' + maxTxt + '</div>'));
										triggerChange = false;
										that.$element.trigger('maxReached' + EVENT_KEY);
									}

									if (maxOptionsGrp && maxReachedGrp) {
										$notify.append($('<div>' + maxTxtGrp + '</div>'));
										triggerChange = false;
										that.$element.trigger('maxReachedGrp' + EVENT_KEY);
									}

									setTimeout(function () {
										that.setSelected(clickedIndex, false);
									}, 10);

									$notify.delay(750).fadeOut(300, function () {
										$(this).remove();
									});
								}
							}
						}
					}

					if (!that.multiple || (that.multiple && that.options.maxOptions === 1)) {
						that.$button.trigger('focus');
					} else if (that.options.liveSearch) {
						that.$searchbox.trigger('focus');
					}

					// Trigger select 'change'
					if (triggerChange) {
						if ((prevValue != getSelectValues(that.$element[0]) && that.multiple) || (prevIndex != that.$element.prop('selectedIndex') && !that.multiple)) {
							// $option.prop('selected') is current option state (selected/unselected). prevValue is the value of the select prior to being changed.
							changedArguments = [option.index, $option.prop('selected'), prevValue];
							that.$element
								.triggerNative('change');
						}
					}
				}
			});

			this.$menu.on('click', 'li.' + classNames.DISABLED + ' a, .' + classNames.POPOVERHEADER + ', .' + classNames.POPOVERHEADER + ' :not(.close)', function (e) {
				if (e.currentTarget == this) {
					e.preventDefault();
					e.stopPropagation();
					if (that.options.liveSearch && !$(e.target).hasClass('close')) {
						that.$searchbox.trigger('focus');
					} else {
						that.$button.trigger('focus');
					}
				}
			});

			this.$menuInner.on('click', '.divider, .dropdown-header', function (e) {
				e.preventDefault();
				e.stopPropagation();
				if (that.options.liveSearch) {
					that.$searchbox.trigger('focus');
				} else {
					that.$button.trigger('focus');
				}
			});

			this.$menu.on('click', '.' + classNames.POPOVERHEADER + ' .close', function () {
				that.$button.trigger('click');
			});

			this.$searchbox.on('click', function (e) {
				e.stopPropagation();
			});

			this.$menu.on('click', '.actions-btn', function (e) {
				if (that.options.liveSearch) {
					that.$searchbox.trigger('focus');
				} else {
					that.$button.trigger('focus');
				}

				e.preventDefault();
				e.stopPropagation();

				if ($(this).hasClass('bs-select-all')) {
					that.selectAll();
				} else {
					that.deselectAll();
				}
			});

			this.$element
				.on('change' + EVENT_KEY, function () {
					that.render();
					that.$element.trigger('changed' + EVENT_KEY, changedArguments);
					changedArguments = null;
				})
				.on('focus' + EVENT_KEY, function () {
					if (!that.options.mobile) that.$button.trigger('focus');
				});
		},

		liveSearchListener: function () {
			var that = this,
					noResults = document.createElement('li');

			this.$button.on('click.bs.dropdown.data-api', function () {
				if (!!that.$searchbox.val()) {
					that.$searchbox.val('');
				}
			});

			this.$searchbox.on('click.bs.dropdown.data-api focus.bs.dropdown.data-api touchend.bs.dropdown.data-api', function (e) {
				e.stopPropagation();
			});

			this.$searchbox.on('input propertychange', function () {
				var searchValue = that.$searchbox.val();

				that.selectpicker.search.elements = [];
				that.selectpicker.search.data = [];

				if (searchValue) {
					var i,
							searchMatch = [],
							q = searchValue.toUpperCase(),
							cache = {},
							cacheArr = [],
							searchStyle = that._searchStyle(),
							normalizeSearch = that.options.liveSearchNormalize;

					if (normalizeSearch) q = normalizeToBase(q);

					that._$lisSelected = that.$menuInner.find('.selected');

					for (var i = 0; i < that.selectpicker.main.data.length; i++) {
						var li = that.selectpicker.main.data[i];

						if (!cache[i]) {
							cache[i] = stringSearch(li, q, searchStyle, normalizeSearch);
						}

						if (cache[i] && li.headerIndex !== undefined && cacheArr.indexOf(li.headerIndex) === -1) {
							if (li.headerIndex > 0) {
								cache[li.headerIndex - 1] = true;
								cacheArr.push(li.headerIndex - 1);
							}

							cache[li.headerIndex] = true;
							cacheArr.push(li.headerIndex);

							cache[li.lastIndex + 1] = true;
						}

						if (cache[i] && li.type !== 'optgroup-label') cacheArr.push(i);
					}

					for (var i = 0, cacheLen = cacheArr.length; i < cacheLen; i++) {
						var index = cacheArr[i],
								prevIndex = cacheArr[i - 1],
								li = that.selectpicker.main.data[index],
								liPrev = that.selectpicker.main.data[prevIndex];

						if (li.type !== 'divider' || (li.type === 'divider' && liPrev && liPrev.type !== 'divider' && cacheLen - 1 !== i)) {
							that.selectpicker.search.data.push(li);
							searchMatch.push(that.selectpicker.main.elements[index]);
						}
					}

					that.activeIndex = undefined;
					that.noScroll = true;
					that.$menuInner.scrollTop(0);
					that.selectpicker.search.elements = searchMatch;
					that.createView(true);

					if (!searchMatch.length) {
						noResults.className = 'no-results';
						noResults.innerHTML = that.options.noneResultsText.replace('{0}', '"' + htmlEscape(searchValue) + '"');
						that.$menuInner[0].firstChild.appendChild(noResults);
					}
				} else {
					that.$menuInner.scrollTop(0);
					that.createView(false);
				}
			});
		},

		_searchStyle: function () {
			return this.options.liveSearchStyle || 'contains';
		},

		val: function (value) {
			if (typeof value !== 'undefined') {
				var prevValue = getSelectValues(this.$element[0]);

				changedArguments = [null, null, prevValue];

				this.$element
					.val(value)
					.trigger('changed' + EVENT_KEY, changedArguments);

				this.render();

				changedArguments = null;

				return this.$element;
			} else {
				return this.$element.val();
			}
		},

		changeAll: function (status) {
			if (!this.multiple) return;
			if (typeof status === 'undefined') status = true;

			var element = this.$element[0],
					previousSelected = 0,
					currentSelected = 0,
					prevValue = getSelectValues(element);

			element.classList.add('bs-select-hidden');

			for (var i = 0, len = this.selectpicker.current.elements.length; i < len; i++) {
				var liData = this.selectpicker.current.data[i],
						option = liData.option;

				if (option && !liData.disabled && liData.type !== 'divider') {
					if (liData.selected) previousSelected++;
					option.selected = status;
					if (status) currentSelected++;
				}
			}

			element.classList.remove('bs-select-hidden');

			if (previousSelected === currentSelected) return;

			this.setOptionStatus();

			this.togglePlaceholder();

			changedArguments = [null, null, prevValue];

			this.$element
				.triggerNative('change');
		},

		selectAll: function () {
			return this.changeAll(true);
		},

		deselectAll: function () {
			return this.changeAll(false);
		},

		toggle: function (e) {
			e = e || window.event;

			if (e) e.stopPropagation();

			this.$button.trigger('click.bs.dropdown.data-api');
		},

		keydown: function (e) {
			var $this = $(this),
					isToggle = $this.hasClass('dropdown-toggle'),
					$parent = isToggle ? $this.closest('.dropdown') : $this.closest(Selector.MENU),
					that = $parent.data('this'),
					$items = that.findLis(),
					index,
					isActive,
					liActive,
					activeLi,
					offset,
					updateScroll = false,
					downOnTab = e.which === keyCodes.TAB && !isToggle && !that.options.selectOnTab,
					isArrowKey = REGEXP_ARROW.test(e.which) || downOnTab,
					scrollTop = that.$menuInner[0].scrollTop,
					isVirtual = that.isVirtual(),
					position0 = isVirtual === true ? that.selectpicker.view.position0 : 0;

			isActive = that.$newElement.hasClass(classNames.SHOW);

			if (
				!isActive &&
				(
					isArrowKey ||
					(e.which >= 48 && e.which <= 57) ||
					(e.which >= 96 && e.which <= 105) ||
					(e.which >= 65 && e.which <= 90)
				)
			) {
				that.$button.trigger('click.bs.dropdown.data-api');

				if (that.options.liveSearch) {
					that.$searchbox.trigger('focus');
					return;
				}
			}

			if (e.which === keyCodes.ESCAPE && isActive) {
				e.preventDefault();
				that.$button.trigger('click.bs.dropdown.data-api').trigger('focus');
			}

			if (isArrowKey) { // if up or down
				if (!$items.length) return;

				// $items.index/.filter is too slow with a large list and no virtual scroll
				index = isVirtual === true ? $items.index($items.filter('.active')) : that.activeIndex;

				if (index === undefined) index = -1;

				if (index !== -1) {
					liActive = that.selectpicker.current.elements[index + position0];
					liActive.classList.remove('active');
					if (liActive.firstChild) liActive.firstChild.classList.remove('active');
				}

				if (e.which === keyCodes.ARROW_UP) { // up
					if (index !== -1) index--;
					if (index + position0 < 0) index += $items.length;

					if (!that.selectpicker.view.canHighlight[index + position0]) {
						index = that.selectpicker.view.canHighlight.slice(0, index + position0).lastIndexOf(true) - position0;
						if (index === -1) index = $items.length - 1;
					}
				} else if (e.which === keyCodes.ARROW_DOWN || downOnTab) { // down
					index++;
					if (index + position0 >= that.selectpicker.view.canHighlight.length) index = 0;

					if (!that.selectpicker.view.canHighlight[index + position0]) {
						index = index + 1 + that.selectpicker.view.canHighlight.slice(index + position0 + 1).indexOf(true);
					}
				}

				e.preventDefault();

				var liActiveIndex = position0 + index;

				if (e.which === keyCodes.ARROW_UP) { // up
					// scroll to bottom and highlight last option
					if (position0 === 0 && index === $items.length - 1) {
						that.$menuInner[0].scrollTop = that.$menuInner[0].scrollHeight;

						liActiveIndex = that.selectpicker.current.elements.length - 1;
					} else {
						activeLi = that.selectpicker.current.data[liActiveIndex];
						offset = activeLi.position - activeLi.height;

						updateScroll = offset < scrollTop;
					}
				} else if (e.which === keyCodes.ARROW_DOWN || downOnTab) { // down
					// scroll to top and highlight first option
					if (index === 0) {
						that.$menuInner[0].scrollTop = 0;

						liActiveIndex = 0;
					} else {
						activeLi = that.selectpicker.current.data[liActiveIndex];
						offset = activeLi.position - that.sizeInfo.menuInnerHeight;

						updateScroll = offset > scrollTop;
					}
				}

				liActive = that.selectpicker.current.elements[liActiveIndex];

				if (liActive) {
					liActive.classList.add('active');
					if (liActive.firstChild) liActive.firstChild.classList.add('active');
				}

				that.activeIndex = that.selectpicker.current.data[liActiveIndex].index;

				that.selectpicker.view.currentActive = liActive;

				if (updateScroll) that.$menuInner[0].scrollTop = offset;

				if (that.options.liveSearch) {
					that.$searchbox.trigger('focus');
				} else {
					$this.trigger('focus');
				}
			} else if (
				(!$this.is('input') && !REGEXP_TAB_OR_ESCAPE.test(e.which)) ||
				(e.which === keyCodes.SPACE && that.selectpicker.keydown.keyHistory)
			) {
				var searchMatch,
						matches = [],
						keyHistory;

				e.preventDefault();

				that.selectpicker.keydown.keyHistory += keyCodeMap[e.which];

				if (that.selectpicker.keydown.resetKeyHistory.cancel) clearTimeout(that.selectpicker.keydown.resetKeyHistory.cancel);
				that.selectpicker.keydown.resetKeyHistory.cancel = that.selectpicker.keydown.resetKeyHistory.start();

				keyHistory = that.selectpicker.keydown.keyHistory;

				// if all letters are the same, set keyHistory to just the first character when searching
				if (/^(.)\1+$/.test(keyHistory)) {
					keyHistory = keyHistory.charAt(0);
				}

				// find matches
				for (var i = 0; i < that.selectpicker.current.data.length; i++) {
					var li = that.selectpicker.current.data[i],
							hasMatch;

					hasMatch = stringSearch(li, keyHistory, 'startsWith', true);

					if (hasMatch && that.selectpicker.view.canHighlight[i]) {
						matches.push(li.index);
					}
				}

				if (matches.length) {
					var matchIndex = 0;

					$items.removeClass('active').find('a').removeClass('active');

					// either only one key has been pressed or they are all the same key
					if (keyHistory.length === 1) {
						matchIndex = matches.indexOf(that.activeIndex);

						if (matchIndex === -1 || matchIndex === matches.length - 1) {
							matchIndex = 0;
						} else {
							matchIndex++;
						}
					}

					searchMatch = matches[matchIndex];

					activeLi = that.selectpicker.main.data[searchMatch];

					if (scrollTop - activeLi.position > 0) {
						offset = activeLi.position - activeLi.height;
						updateScroll = true;
					} else {
						offset = activeLi.position - that.sizeInfo.menuInnerHeight;
						// if the option is already visible at the current scroll position, just keep it the same
						updateScroll = activeLi.position > scrollTop + that.sizeInfo.menuInnerHeight;
					}

					liActive = that.selectpicker.main.elements[searchMatch];
					liActive.classList.add('active');
					if (liActive.firstChild) liActive.firstChild.classList.add('active');
					that.activeIndex = matches[matchIndex];

					liActive.firstChild.focus();

					if (updateScroll) that.$menuInner[0].scrollTop = offset;

					$this.trigger('focus');
				}
			}

			// Select focused option if "Enter", "Spacebar" or "Tab" (when selectOnTab is true) are pressed inside the menu.
			if (
				isActive &&
				(
					(e.which === keyCodes.SPACE && !that.selectpicker.keydown.keyHistory) ||
					e.which === keyCodes.ENTER ||
					(e.which === keyCodes.TAB && that.options.selectOnTab)
				)
			) {
				if (e.which !== keyCodes.SPACE) e.preventDefault();

				if (!that.options.liveSearch || e.which !== keyCodes.SPACE) {
					that.$menuInner.find('.active a').trigger('click', true); // retain active class
					$this.trigger('focus');

					if (!that.options.liveSearch) {
						// Prevent screen from scrolling if the user hits the spacebar
						e.preventDefault();
						// Fixes spacebar selection of dropdown items in FF & IE
						$(document).data('spaceSelect', true);
					}
				}
			}
		},

		mobile: function () {
			this.$element[0].classList.add('mobile-device');
		},

		refresh: function () {
			// update options if data attributes have been changed
			var config = $.extend({}, this.options, this.$element.data());
			this.options = config;

			this.checkDisabled();
			this.setStyle();
			this.render();
			this.createLi();
			this.setWidth();

			this.setSize(true);

			this.$element.trigger('refreshed' + EVENT_KEY);
		},

		hide: function () {
			this.$newElement.hide();
		},

		show: function () {
			this.$newElement.show();
		},

		remove: function () {
			this.$newElement.remove();
			this.$element.remove();
		},

		destroy: function () {
			this.$newElement.before(this.$element).remove();

			if (this.$bsContainer) {
				this.$bsContainer.remove();
			} else {
				this.$menu.remove();
			}

			this.$element
				.off(EVENT_KEY)
				.removeData('selectpicker')
				.removeClass('bs-select-hidden selectpicker');

			$(window).off(EVENT_KEY + '.' + this.selectId);
		}
	};

	// SELECTPICKER PLUGIN DEFINITION
	// ==============================
	function Plugin (option) {
		// get the args of the outer function..
		var args = arguments;
		// The arguments of the function are explicitly re-defined from the argument list, because the shift causes them
		// to get lost/corrupted in android 2.3 and IE9 #715 #775
		var _option = option;

		[].shift.apply(args);

		// if the version was not set successfully
		if (!version.success) {
			// try to retreive it again
			try {
				version.full = ($.fn.dropdown.Constructor.VERSION || '').split(' ')[0].split('.');
			} catch (err) {
				// fall back to use BootstrapVersion if set
				if (Selectpicker.BootstrapVersion) {
					version.full = Selectpicker.BootstrapVersion.split(' ')[0].split('.');
				} else {
					version.full = [version.major, '0', '0'];

					console.warn(
						'There was an issue retrieving Bootstrap\'s version. ' +
						'Ensure Bootstrap is being loaded before bootstrap-select and there is no namespace collision. ' +
						'If loading Bootstrap asynchronously, the version may need to be manually specified via $.fn.selectpicker.Constructor.BootstrapVersion.',
						err
					);
				}
			}

			version.major = version.full[0];
			version.success = true;
		}

		if (version.major === '4') {
			// some defaults need to be changed if using Bootstrap 4
			// check to see if they have already been manually changed before forcing them to update
			var toUpdate = [];

			if (Selectpicker.DEFAULTS.style === classNames.BUTTONCLASS) toUpdate.push({ name: 'style', className: 'BUTTONCLASS' });
			if (Selectpicker.DEFAULTS.iconBase === classNames.ICONBASE) toUpdate.push({ name: 'iconBase', className: 'ICONBASE' });
			if (Selectpicker.DEFAULTS.tickIcon === classNames.TICKICON) toUpdate.push({ name: 'tickIcon', className: 'TICKICON' });

			classNames.DIVIDER = 'dropdown-divider';
			classNames.SHOW = 'show';
			classNames.BUTTONCLASS = 'btn-light';
			classNames.POPOVERHEADER = 'popover-header';
			classNames.ICONBASE = '';
			classNames.TICKICON = 'bs-ok-default';

			for (var i = 0; i < toUpdate.length; i++) {
				var option = toUpdate[i];
				Selectpicker.DEFAULTS[option.name] = classNames[option.className];
			}
		}

		var value;
		var chain = this.each(function () {
			var $this = $(this);
			if ($this.is('select')) {
				var data = $this.data('selectpicker'),
						options = typeof _option == 'object' && _option;

				if (!data) {
					var dataAttributes = $this.data();

					for (var dataAttr in dataAttributes) {
						if (dataAttributes.hasOwnProperty(dataAttr) && $.inArray(dataAttr, DISALLOWED_ATTRIBUTES) !== -1) {
							delete dataAttributes[dataAttr];
						}
					}

					var config = $.extend({}, Selectpicker.DEFAULTS, $.fn.selectpicker.defaults || {}, dataAttributes, options);
					config.template = $.extend({}, Selectpicker.DEFAULTS.template, ($.fn.selectpicker.defaults ? $.fn.selectpicker.defaults.template : {}), dataAttributes.template, options.template);
					$this.data('selectpicker', (data = new Selectpicker(this, config)));
				} else if (options) {
					for (var i in options) {
						if (options.hasOwnProperty(i)) {
							data.options[i] = options[i];
						}
					}
				}

				if (typeof _option == 'string') {
					if (data[_option] instanceof Function) {
						value = data[_option].apply(data, args);
					} else {
						value = data.options[_option];
					}
				}
			}
		});

		if (typeof value !== 'undefined') {
			// noinspection JSUnusedAssignment
			return value;
		} else {
			return chain;
		}
	}

	var old = $.fn.selectpicker;
	$.fn.selectpicker = Plugin;
	$.fn.selectpicker.Constructor = Selectpicker;

	// SELECTPICKER NO CONFLICT
	// ========================
	$.fn.selectpicker.noConflict = function () {
		$.fn.selectpicker = old;
		return this;
	};

	$(document)
		.off('keydown.bs.dropdown.data-api')
		.on('keydown' + EVENT_KEY, '.bootstrap-select [data-toggle="dropdown"], .bootstrap-select [role="listbox"], .bootstrap-select .bs-searchbox input', Selectpicker.prototype.keydown)
		.on('focusin.modal', '.bootstrap-select [data-toggle="dropdown"], .bootstrap-select [role="listbox"], .bootstrap-select .bs-searchbox input', function (e) {
			e.stopPropagation();
		});

	// SELECTPICKER DATA-API
	// =====================
	$(window).on('load' + EVENT_KEY + '.data-api', function () {
		$('.selectpicker').each(function () {
			var $selectpicker = $(this);
			Plugin.call($selectpicker, $selectpicker.data());
		})
	});
})(jQuery);
}));

/******************************************************************************/

// https://github.com/jackmoore/autosize
(function (global, factory) {
	if (typeof define === "function" && define.amd) {
		define(['module', 'exports'], factory);
	} else if (typeof exports !== "undefined") {
		factory(module, exports);
	} else {
		var mod = {
			exports: {}
		};
		factory(mod, mod.exports);
		global.autosize = mod.exports;
	}
})(this, function (module, exports) {
	'use strict';

	var map = typeof Map === "function" ? new Map() : function () {
		var keys = [];
		var values = [];

		return {
			has: function has(key) {
				return keys.indexOf(key) > -1;
			},
			get: function get(key) {
				return values[keys.indexOf(key)];
			},
			set: function set(key, value) {
				if (keys.indexOf(key) === -1) {
					keys.push(key);
					values.push(value);
				}
			},
			delete: function _delete(key) {
				var index = keys.indexOf(key);
				if (index > -1) {
					keys.splice(index, 1);
					values.splice(index, 1);
				}
			}
		};
	}();

	var createEvent = function createEvent(name) {
		return new Event(name, { bubbles: true });
	};
	try {
		new Event('test');
	} catch (e) {
		// IE does not support 'new Event()'
		createEvent = function createEvent(name) {
			var evt = document.createEvent('Event');
			evt.initEvent(name, true, false);
			return evt;
		};
	}

	function assign(ta) {
		if (!ta || !ta.nodeName || ta.nodeName !== 'TEXTAREA' || map.has(ta)) return;

		var heightOffset = null;
		var clientWidth = null;
		var cachedHeight = null;

		function init() {
			var style = window.getComputedStyle(ta, null);

			if (style.resize === 'vertical') {
				ta.style.resize = 'none';
			} else if (style.resize === 'both') {
				ta.style.resize = 'horizontal';
			}

			if (style.boxSizing === 'content-box') {
				heightOffset = -(parseFloat(style.paddingTop) + parseFloat(style.paddingBottom));
			} else {
				heightOffset = parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);
			}
			// Fix when a textarea is not on document body and heightOffset is Not a Number
			if (isNaN(heightOffset)) {
				heightOffset = 0;
			}

			update();
		}

		function changeOverflow(value) {
			{
				// Chrome/Safari-specific fix:
				// When the textarea y-overflow is hidden, Chrome/Safari do not reflow the text to account for the space
				// made available by removing the scrollbar. The following forces the necessary text reflow.
				var width = ta.style.width;
				ta.style.width = '0px';
				// Force reflow:
				/* jshint ignore:start */
				ta.offsetWidth;
				/* jshint ignore:end */
				ta.style.width = width;
			}

			ta.style.overflowY = value;
		}

		function getParentOverflows(el) {
			var arr = [];

			while (el && el.parentNode && el.parentNode instanceof Element) {
				if (el.parentNode.scrollTop) {
					arr.push({
						node: el.parentNode,
						scrollTop: el.parentNode.scrollTop
					});
				}
				el = el.parentNode;
			}

			return arr;
		}

		function resize() {
			if (ta.scrollHeight === 0) {
				// If the scrollHeight is 0, then the element probably has display:none or is detached from the DOM.
				return;
			}

			var overflows = getParentOverflows(ta);
			var docTop = document.documentElement && document.documentElement.scrollTop; // Needed for Mobile IE (ticket #240)

			ta.style.height = '';
			ta.style.height = ta.scrollHeight + heightOffset + 'px';

			// used to check if an update is actually necessary on window.resize
			clientWidth = ta.clientWidth;

			// prevents scroll-position jumping
			overflows.forEach(function (el) {
				el.node.scrollTop = el.scrollTop;
			});

			if (docTop) {
				document.documentElement.scrollTop = docTop;
			}
		}

		function update() {
			resize();

			var styleHeight = Math.round(parseFloat(ta.style.height));
			var computed = window.getComputedStyle(ta, null);

			// Using offsetHeight as a replacement for computed.height in IE, because IE does not account use of border-box
			var actualHeight = computed.boxSizing === 'content-box' ? Math.round(parseFloat(computed.height)) : ta.offsetHeight;

			// The actual height not matching the style height (set via the resize method) indicates that 
			// the max-height has been exceeded, in which case the overflow should be allowed.
			if (actualHeight < styleHeight) {
				if (computed.overflowY === 'hidden') {
					changeOverflow('scroll');
					resize();
					actualHeight = computed.boxSizing === 'content-box' ? Math.round(parseFloat(window.getComputedStyle(ta, null).height)) : ta.offsetHeight;
				}
			} else {
				// Normally keep overflow set to hidden, to avoid flash of scrollbar as the textarea expands.
				if (computed.overflowY !== 'hidden') {
					changeOverflow('hidden');
					resize();
					actualHeight = computed.boxSizing === 'content-box' ? Math.round(parseFloat(window.getComputedStyle(ta, null).height)) : ta.offsetHeight;
				}
			}

			if (cachedHeight !== actualHeight) {
				cachedHeight = actualHeight;
				var evt = createEvent('autosize:resized');
				try {
					ta.dispatchEvent(evt);
				} catch (err) {
					// Firefox will throw an error on dispatchEvent for a detached element
					// https://bugzilla.mozilla.org/show_bug.cgi?id=889376
				}
			}
		}

		var pageResize = function pageResize() {
			if (ta.clientWidth !== clientWidth) {
				update();
			}
		};

		var destroy = function (style) {
			window.removeEventListener('resize', pageResize, false);
			ta.removeEventListener('input', update, false);
			ta.removeEventListener('keyup', update, false);
			ta.removeEventListener('autosize:destroy', destroy, false);
			ta.removeEventListener('autosize:update', update, false);

			Object.keys(style).forEach(function (key) {
				ta.style[key] = style[key];
			});

			map.delete(ta);
		}.bind(ta, {
			height: ta.style.height,
			resize: ta.style.resize,
			overflowY: ta.style.overflowY,
			overflowX: ta.style.overflowX,
			wordWrap: ta.style.wordWrap
		});

		ta.addEventListener('autosize:destroy', destroy, false);

		// IE9 does not fire onpropertychange or oninput for deletions,
		// so binding to onkeyup to catch most of those events.
		// There is no way that I know of to detect something like 'cut' in IE9.
		if ('onpropertychange' in ta && 'oninput' in ta) {
			ta.addEventListener('keyup', update, false);
		}

		window.addEventListener('resize', pageResize, false);
		ta.addEventListener('input', update, false);
		ta.addEventListener('autosize:update', update, false);
		ta.style.overflowX = 'hidden';
		ta.style.wordWrap = 'break-word';

		map.set(ta, {
			destroy: destroy,
			update: update
		});

		init();
	}

	function destroy(ta) {
		var methods = map.get(ta);
		if (methods) {
			methods.destroy();
		}
	}

	function update(ta) {
		var methods = map.get(ta);
		if (methods) {
			methods.update();
		}
	}

	var autosize = null;

	// Do nothing in Node.js environment and IE8 (or lower)
	if (typeof window === 'undefined' || typeof window.getComputedStyle !== 'function') {
		autosize = function autosize(el) {
			return el;
		};
		autosize.destroy = function (el) {
			return el;
		};
		autosize.update = function (el) {
			return el;
		};
	} else {
		autosize = function autosize(el, options) {
			if (el) {
				Array.prototype.forEach.call(el.length ? el : [el], function (x) {
					return assign(x, options);
				});
			}
			return el;
		};
		autosize.destroy = function (el) {
			if (el) {
				Array.prototype.forEach.call(el.length ? el : [el], destroy);
			}
			return el;
		};
		autosize.update = function (el) {
			if (el) {
				Array.prototype.forEach.call(el.length ? el : [el], update);
			}
			return el;
		};
	}

	exports.default = autosize;
	module.exports = exports['default'];
});

/******************************************************************************/

(function(window, $) {
	var fave = function(window, $) {
		// Private
		var FormDataWasChanged = false;

		function IsDebugMode() {
			return window.fave_debug && window.fave_debug === true;
		};

		function GetModalAlertTmpl(title, message, error) {
			return '<div class="alert alert-' + (!error?'success':'danger') + ' alert-dismissible fade show" role="alert"><strong>' + title + '</strong> ' + message + '<button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button></div>';
		};

		function ShowSystemMsg(title, message, error) {
			var modal_alert_place = $('.modal.show .sys-messages');
			if(!modal_alert_place.length) {
				modal_alert_place = $('form.alert-here .sys-messages');
			}
			if(modal_alert_place.length) {
				modal_alert_place.html(GetModalAlertTmpl(title, message, error));
			} else {
				ShowSystemMsgModal(title, message, error);
			}
		};

		function ShowSystemMsgModal(title, message, error) {
			$('#sys-modal-system-message-placeholder').html('');
			var html = '<div class="modal fade" id="sys-modal-system-message" tabindex="-1" role="dialog" aria-labelledby="sysModalSystemMessageLabel" aria-hidden="true"> \
				<div class="modal-dialog modal-dialog-centered" role="document"> \
					<div class="modal-content"> \
							<input type="hidden" name="action" value="index-user-update-profile"> \
							<div class="modal-header"> \
								<h5 class="modal-title" id="sysModalSystemMessageLabel">' + title + '</h5> \
								<button type="button" class="close" data-dismiss="modal" aria-label="Close"> \
									<span aria-hidden="true">&times;</span> \
								</button> \
							</div> \
							<div class="modal-body text-left">' + message + '</div> \
							<div class="modal-footer"> \
								<button type="button" class="btn btn-secondary" data-dismiss="modal">Cancel</button> \
							</div> \
					</div> \
				</div> \
			</div>';
			$('#sys-modal-system-message-placeholder').html(html);
			$('#sys-modal-system-message').modal({
				backdrop: 'static',
				keyboard: true,
				show: false,
			});
			$('#sys-modal-system-message').on('hidden.bs.modal', function(e) {
				$('#sys-modal-system-message-placeholder').html('');
			});
			$('#sys-modal-system-message').modal('show');
		};

		function AjaxEval(data) {
			try {
				eval(data);
			} catch(e) {
				if(e instanceof SyntaxError) {
					console.log(data);
					console.log('Error: JavaScript code eval error', e.message);
				}
			}
		};

		function AjaxDone(data) {
			AjaxEval(data);
		};

		function AjaxFail(data, status, error) {
			if(status.toLowerCase() === "error" && error.toLowerCase() === "not found") {
				AjaxEval(data);
			} else {
				console.log('Error: data sending error, page will be reloaded', data, status, error);
				setTimeout(function() {
					window.location.reload(false);
				}, 1000);
			}
		};

		function FormToAjax(form) {
			form.submit(function(e) {
				if(form.hasClass('loading')) {
					e.preventDefault();
					return;
				}

				// Block send button
				form.addClass('loading').addClass('alert-here');
				var button = form.find('button[type=submit]');
				button.addClass('progress-bar-striped')
					.addClass('progress-bar-animated');

				// Another button
				if(button.attr('data-target') != '') {
					$('#' + button.attr('data-target')).addClass('progress-bar-striped')
						.addClass('progress-bar-animated');
				}

				// Clear form messages
				form.find('.sys-messages').html('');

				$.ajax({
					type: "POST",
					url: form.attr('action'),
					data: form.serialize()
				}).done(function(data) {
					FormDataWasChanged = false;
					if(IsDebugMode()) console.log('done', data);
					AjaxDone(data)
				}).fail(function(xhr, status, error) {
					if(IsDebugMode()) console.log('fail', xhr, status, error);
					AjaxFail(xhr.responseText, status, error);
				}).always(function() {
					// Add delay for one second
					setTimeout(function() {
						form.removeClass('loading').removeClass('alert-here');
						button.removeClass('progress-bar-striped')
							.removeClass('progress-bar-animated');
						// Another button
						if(button.attr('data-target') != '') {
							$('#' + button.attr('data-target'))
								.removeClass('progress-bar-striped')
								.removeClass('progress-bar-animated');
						}
					}, 100);
				});

				// Prevent submit action
				e.preventDefault();
			});

			// Bind to another button
			var button = form.find('button[type=submit]');
			if(button.attr('data-target') != '') {
				$('#' + button.attr('data-target')).click(function() {
					button.click();
				});
			}

			// Mark body if any data in form was changed
			if(form.hasClass('prev-data-lost')) {
				form.find('input, textarea, select').on('input', function() {
					if(!FormDataWasChanged) {
						if(!$(this).hasClass('ignore-lost-data')) {
							FormDataWasChanged = true;
						}
					}
				});
			}
		};

		function PreventDataLost() {
			if(!FormDataWasChanged) {
				FormDataWasChanged = true;
			}
		};

		function FormDataIsChanged() {
			return FormDataWasChanged;
		}

		function HtmlDecode(value) {
			var doc = new DOMParser().parseFromString(value, "text/html");
			return doc.documentElement.textContent;
		};

		function HtmlFixEditorHtml(value) {
			newValue = value;
			newValue = newValue.replace(/&nbsp;/gi, '');
			return newValue;
		};

		function AllFormsToAjax() {
			$('form').each(function() {
				FormToAjax($(this));
			});
		};

		function BindWindowBeforeUnload() {
			// Prevent page reload if data was changed
			$(window).bind('beforeunload', function(){
				if(FormDataWasChanged) {
					return 'Some data was changed and not saved. Are you sure want to leave page?';
				}
			});
		};

		function MakeTextAreasAutoSized() {
			autosize($('textarea.autosize'));
		};

		function MakeTextAreasWysiwyg() {
			$('textarea.wysiwyg').each(function() {
				var area = $(this)[0];
				var area_id = area.id;
				var area_name = area.name;
				var area_html = area.innerHTML;

				// Wrap editro by additional DIV and remove target textarea
				$(area).wrap('<div id="' + area_id + '_wysiwyg" class="form-control wysiwyg" style="height:auto;padding:0px"></div>').remove();

				var wysiwyg = document.getElementById(area_id + '_wysiwyg');
				wysiwyg.id = area_id;

				// Create and init editor
				var editor = window.pell.init({
					element: wysiwyg,
					onChange: function(html) {
						area.innerHTML = HtmlFixEditorHtml(html);
						$(area).val(HtmlFixEditorHtml(html));
						if(!FormDataWasChanged) {
							FormDataWasChanged = true;
						}
					},
					defaultParagraphSeparator: 'p',
					styleWithCSS: false,
					actions: [
						'paragraph',
						'heading1',
						'heading2',
						'bold',
						'italic',
						'underline',
						'strikethrough',
						'ulist',
						'olist',
						'link',
						{
							name: 'htmlcode',
							icon: 'HTML',
							title: 'HTML Source',
							result: function(edt, ctn, btn) {
								var jedt = $(edt);
								var jctn = $(ctn);
								var jbtn = $(btn);
								if(!jbtn.hasClass('pell-button-html-pressed')) {
									jbtn.addClass('pell-button-html-pressed');
									jedt.addClass('pell-html-mode');

									jedt.find('.pell-actionbar .pell-button').prop('disabled', true);
									jbtn.prop('disabled', false);

									setTimeout(function() {
										jedt.find('textarea.form-control').focus();
									}, 0);
								} else {
									jbtn.removeClass('pell-button-html-pressed');
									jedt.removeClass('pell-html-mode');
									jedt.find('.pell-actionbar .pell-button').prop('disabled', false);

									var srcValue = jedt.find('textarea.form-control').val();
									ctn.innerHTML = HtmlFixEditorHtml(srcValue);
									$(ctn).val(HtmlFixEditorHtml(srcValue));

									setTimeout(function() {
										jctn.focus();
									}, 0);
								}
							},
						},
					],
					classes: {
						actionbar: 'pell-actionbar',
						button: 'pell-button',
						content: 'pell-content',
						selected: 'pell-button-selected'
					}
				});

				editor.onfocusin = function() {
					$(wysiwyg).addClass('focused');
				};

				editor.onfocusout = function() {
					$(wysiwyg).find('.pell-actionbar button.pell-button-selected').removeClass('pell-button-selected');
					$(wysiwyg).removeClass('focused');
				};

				// Re-add textarea
				$(wysiwyg).append('<textarea class="form-control" id="' + area_id + '_wysiwyg' + '" name="' + area_name + '" style="display:none"></textarea>');
				area = document.getElementById(area_id + '_wysiwyg');

				// Prevent data lost if HTML was changed
				$(area).on('input', function() {
					if(!FormDataWasChanged) {
						FormDataWasChanged = true;
					}
				});

				// Copy HTML to textarea and editor
				area.innerHTML = HtmlFixEditorHtml(HtmlDecode(area_html));
				$(area).val(HtmlFixEditorHtml(HtmlDecode(area_html)));
				editor.content.innerHTML = HtmlFixEditorHtml(HtmlDecode(area_html));
			});
		};

		function MakeTextAreasTmplEditor() {
			var IgnoreDataLost = true;
			$('textarea.tmpl-editor').each(function() {
				var targetTextArea = $(this)[0];
				var targetFileExt = $(this).data('emode');
				var targetEditorMode = 'text/html';
				if(targetFileExt == 'js') {
					targetEditorMode = 'javascript';
				} else if(targetFileExt == 'css') {
					targetEditorMode = 'css';
				}
				CodeMirror.fromTextArea(targetTextArea, {
					lineNumbers: true,
					lineWrapping: true,
					viewportMargin: Infinity,
					mode: targetEditorMode,
				}).on('change', function(editor){
					targetTextArea.value = editor.getValue();
					if(!IgnoreDataLost) {
						if(!FormDataWasChanged) {
							FormDataWasChanged = true;
						}
					}
				});
			});
			IgnoreDataLost = false;
		};

		function MakeTextAreasNotReactOnTab() {
			$('textarea.use-tab-key').each(function() {
				$(this).keydown(function(e) {
					if(e.keyCode === 9) {
						var start = this.selectionStart;
						var end = this.selectionEnd;
						var $this = $(this);
						var value = $this.val();
						$this.val(value.substring(0, start) + "\t" + value.substring(end));
						this.selectionStart = this.selectionEnd = start + 1;
						e.preventDefault();
						if(!FormDataWasChanged) {
							FormDataWasChanged = true;
						}
					}
				});
			});
		};

		function Initialize() {
			// Check if jQuery was loaded
			if(typeof $ == 'function') {
				AllFormsToAjax();
				BindWindowBeforeUnload();
				MakeTextAreasAutoSized();
				MakeTextAreasWysiwyg();
				MakeTextAreasTmplEditor();
				MakeTextAreasNotReactOnTab();
			} else {
				console.log('Error: jQuery is not loaded!');
			}
		};

		// Initialize
		if(window.addEventListener) {
			// W3C standard
			window.addEventListener('load', Initialize, false);
		} else if(window.attachEvent) {
			// Microsoft
			window.attachEvent('onload', Initialize);
		};

		// Public
		return {
			ShowMsgSuccess: function(title, message) {
				ShowSystemMsg(title, message, false);
			},

			ShowMsgError: function(title, message) {
				ShowSystemMsg(title, message, true);
			},

			FormDataWasChanged: function() {
				PreventDataLost();
			},

			ModalUserProfile: function() {
				var html = '<div class="modal fade" id="sys-modal-user-settings" tabindex="-1" role="dialog" aria-labelledby="sysModalUserSettingsLabel" aria-hidden="true"> \
					<div class="modal-dialog modal-dialog-centered" role="document"> \
						<div class="modal-content"> \
							<form class="form-user-settings" action="/cp/" method="post" autocomplete="off"> \
								<input type="hidden" name="action" value="index-user-update-profile"> \
								<div class="modal-header"> \
									<h5 class="modal-title" id="sysModalUserSettingsLabel">My profile</h5> \
									<button type="button" class="close" data-dismiss="modal" aria-label="Close"> \
										<span aria-hidden="true">&times;</span> \
									</button> \
								</div> \
								<div class="modal-body text-left"> \
									<div class="form-group"> \
										<label for="first_name">First name</label> \
										<input type="text" class="form-control" id="first_name" name="first_name" value="' + window.CurrentUserProfileData.first_name + '" placeholder="User first name" autocomplete="off"> \
									</div> \
									<div class="form-group"> \
										<label for="last_name">Last name</label> \
										<input type="text" class="form-control" id="last_name" name="last_name" value="' + window.CurrentUserProfileData.last_name + '" placeholder="User last name" autocomplete="off"> \
									</div> \
									<div class="form-group"> \
										<label for="email">Email</label> \
										<input type="email" class="form-control" id="email" name="email" value="' + window.CurrentUserProfileData.email + '" placeholder="User email" autocomplete="off" required> \
									</div> \
									<div class="form-group"> \
										<label for="password">New password</label> \
										<input type="password" class="form-control" id="password" aria-describedby="passwordHelp" name="password" value="" placeholder="User new password" autocomplete="off"> \
										<small id="passwordHelp" class="form-text text-muted">Leave this field empty if you don\'t want change your password</small> \
									</div> \
									<div class="sys-messages"></div> \
								</div> \
								<div class="modal-footer"> \
									<button type="submit" class="btn btn-primary">Save</button> \
									<button type="button" class="btn btn-secondary" data-dismiss="modal">Cancel</button> \
								</div> \
							</form> \
						</div> \
					</div> \
				</div>';
				$('#sys-modal-user-settings-placeholder').html(html);
				$("#sys-modal-user-settings").modal({
					backdrop: 'static',
					keyboard: false,
					show: false,
				});
				$('#sys-modal-user-settings').on('hidden.bs.modal', function(e) {
					$('#sys-modal-user-settings-placeholder').html('');
				});
				FormToAjax($('#sys-modal-user-settings form'));
				$("#sys-modal-user-settings").modal('show');
			},

			ShopProductsAdd: function() {
				var selText = $('#lbl_attributes option:selected').text();
				var selValue = $('#lbl_attributes').val();
				if(selValue == '0') { return; }
				$('#lbl_attributes')[0].selectedIndex = 0;
				$('#lbl_attributes').selectpicker('refresh');
				if($('#prod_attr_' + selValue).length > 0) { return; }
				$('#list').append('<div class="form-group" id="prod_attr_' + selValue + '"><div><b>' + selText + '</b></div><div class="position-relative"><select class="form-control" name="value.' + selValue + '" autocomplete="off" required disabled><option value="0">Loading values...</option></select><button type="button" class="btn btn-danger btn-dynamic-remove" onclick="fave.ShopProductsRemove(this);" disabled>&times;</button></div></div>');
				PreventDataLost();
				$.ajax({
					type: 'POST',
					url: '/cp/',
					data: {
						action: 'shop-get-attribute-values',
						id: selValue
					}
				}).done(function(data) {
					try {
						eval(data);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(data);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				}).fail(function(xhr, status, error) {
					$('#prod_attr_' + selValue).remove();
					try {
						eval(xhr.responseText);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(xhr.responseText);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				});
			},

			ShopProductsRemove: function(button) {
				$(button).parent().parent().remove();
				PreventDataLost();
			},

			ShopAttributesAdd: function() {
				$('#list').append('<div class="form-group position-relative"><input class="form-control" type="text" name="value.0" value="" placeholder="" autocomplete="off" required><button type="button" class="btn btn-danger btn-dynamic-remove" onclick="fave.ShopAttributesRemove(this);">&times;</button></div>');
				PreventDataLost();
				setTimeout(function() {
					$('#list input').last().focus();
				}, 100);
			},

			ShopAttributesRemove: function(button) {
				$(button).parent().remove();
				PreventDataLost();
			},

			ShopProductsUploadImage: function(action_name, product_id, input_id) {
				var file_el = $('#' + input_id)[0];
				if(!file_el.files) return;
				if(file_el.files.length <= 0) return;

				$('#img-upload-block input').prop('disabled', true);
				$('#upload-msg').css('display', 'block');

				var fd = new FormData();
				fd.append('action', action_name);
				fd.append('id', product_id);
				fd.append('count', file_el.files.length);
				for(var i = 0; i < file_el.files.length; i++) {
					fd.append('file_' + i, file_el.files[i]);
				}

				$.ajax({
					url: '/cp/',
					method: 'POST',
					type: 'POST',
					data: fd,
					contentType: false,
					processData: false
				}).done(function(data) {
					try {
						eval(data);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(data);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				}).fail(function(xhr, status, error) {
					try {
						eval(xhr.responseText);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(xhr.responseText);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				}).always(function() {
					file_el.value = '';
					$('#img-upload-block input').prop('disabled', false);
					$('#upload-msg').css('display', 'none');
				});
			},

			ShopProductsDeleteImage: function(button, product_id, filename) {
				$.ajax({
					type: "POST",
					url: '/cp/',
					data: {
						action: 'shop-upload-delete',
						id: product_id,
						file: filename,
					}
				}).done(function(data) {
					if(IsDebugMode()) console.log('done', data);
					AjaxDone(data);
				}).fail(function(xhr, status, error) {
					if(IsDebugMode()) console.log('fail', xhr, status, error);
					AjaxFail(xhr.responseText, status, error);
				});
			},

			ShopProductsDuplicate: function(button, product_id) {
				if($(button).hasClass('in-progress')) return;
				if(FormDataIsChanged()) {
					fave.ShowMsgError('Warning!', 'Something was changed, save changes before duplicate product', true);
					return;
				}
				$(button).addClass('in-progress');
				$.ajax({
					type: "POST",
					url: '/cp/',
					data: {
						action: 'shop-duplicate',
						id: product_id,
					}
				}).done(function(data) {
					try {
						eval(data);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(data);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				}).fail(function(xhr, status, error) {
					try {
						eval(xhr.responseText);
					} catch(e) {
						if(e instanceof SyntaxError) {
							console.log(xhr.responseText);
							console.log('Error: JavaScript code eval error', e.message);
						}
					}
				}).always(function() {
					$(button).removeClass('in-progress');
				});
			},

			ShopProductsRetryImage: function(img) {
				var original = $(img).attr('src');
				$(img).attr('src', '/assets/cp/img-load.gif');
				setTimeout(function() {
					$(img).attr('src', original);
				}, 1000);
			},

			ActionLogout: function(message) {
				if(confirm(message)) {
					$.ajax({
						type: "POST",
						url: '/cp/',
						data: {
							action: 'index-user-logout',
						}
					}).done(function(data) {
						if(IsDebugMode()) console.log('done', data);
						AjaxDone(data);
					}).fail(function(xhr, status, error) {
						if(IsDebugMode()) console.log('fail', xhr, status, error);
						AjaxFail(xhr.responseText, status, error);
					});
				}
			},

			ActionDataTableDelete: function(object, action, id, message) {
				if(confirm(message)) {
					$.ajax({
						type: "POST",
						url: '/cp/',
						data: {
							action: action,
							id: id,
						}
					}).done(function(data) {
						if(IsDebugMode()) console.log('done', data);
						AjaxDone(data);
					}).fail(function(xhr, status, error) {
						if(IsDebugMode()) console.log('fail', xhr, status, error);
						AjaxFail(xhr.responseText, status, error);
					});
				}
			},

			ActionRestoreThemeFile: function(action_name, file_name, message) {
				if(confirm(message)) {
					$.ajax({
						type: "POST",
						url: '/cp/',
						data: {
							action: action_name,
							file: file_name,
						}
					}).done(function(data) {
						if(IsDebugMode()) console.log('done', data);
						AjaxDone(data);
					}).fail(function(xhr, status, error) {
						if(IsDebugMode()) console.log('fail', xhr, status, error);
						AjaxFail(xhr.responseText, status, error);
					});
				}
				return false;
			},
		};
	}(window, $);

	// Make it public
	window.fave = fave;
}(window, jQuery));